//########################################################################################
//#                                                                       26.04.2023/TDU #
//#         Code snippets for kannmotion sequence in c.                                  #
//#                                                                                      #
//#         Not all snippets/functions are available for all KannMOTION devices.         #
//#         It depends on hardware and firmware version.                                 #
//#                                                                                      #
//########################################################################################
//# Overview:   - Write drive config                                                     #
//#             - Change GoToPos max speed                                               #
//#             - Read drive config                                                      #
//#             - Store nonvolatile user variables                                       #
//#             - Homing on stall                                                        #
//#             - Homing on input                                                        #
//#             - Set 0 position                                                         #
//#             - Set custom position                                                    #
//#             - Go to position with tolarance                                          #
//#             - Rotate                                                                 #
//#             - Fast Stop                                                              #
//#             - Set/Reset digital outputs                                              #
//#             - Check for digital inputs                                               #
//#             - Set/Reset user LED                                                     #
//#             - Analog In/Out                                                          #
//#             - Serial Rx event                                                        #
//#             - Serial Tx function                                                     #
//#             - CAN Rx event                                                           #
//#             - CAN Tx function                                                        #
//#             - Allow/block external move commands                                     #
//#             - Multiple infos (calls/read/variables)                                  #
//########################################################################################

//###########################################
//#           Write drive config            #
//###########################################
    case INIT_CONFIG:
    {
        // Config parameters
        UFu_DrvChgSetting(12, 600);        // Min. Speed	[0.1rpm]
        UFu_DrvChgSetting(13, 2000);       // Max. Speed	[0.1rpm]
        UFu_DrvChgSetting(14, 50);         // Torque-Hold	[0.5%]
        UFu_DrvChgSetting(15, 200);        // Torque-ACC	[0.5%]
        UFu_DrvChgSetting(16, 90);         // Torque-RUN	[0.5%]
        UFu_DrvChgSetting(17, 200);        // Torque-DEC	[0.5%]
        UFu_DrvChgSetting(18, 20);         // Acceleration	[0.5%]
        UFu_DrvChgSetting(19, 200);        // Deceleration	[0.5%]
        UFu_DrvChgSetting(21, 0b00000000); // Filter Din/Ain

        UVar.u8_StepChain[0]++;
        break;
    }
	
	
//###########################################
//#        Change GoToPos max speed         #
//###########################################
//# Reduces internal max speed to a certain #
//# percent value.                          #
//###########################################
    UFu_DrvSetMaxSpeed(160);	// Set max speed to 80% (160*0.5%)


//###########################################
//#            Read drive config            #
//###########################################
#define	u16_minSpeed		UVar.KxxFlexUser.u16_UserVar[0]
#define	u16_maxCurrentmA	UVar.KxxFlexUser.u16_UserVar[1]
// ...
    case READ_DRIVE_CONFIG:
    {
        tMOTDRV_PARAM testParam;
        
        testParam.u8_PID = eU16_MinSpeed_rpm;	// from tEN_SYSPARAM_ID
        if(UFu_GetDrvParam(&testParam) == eMS_OK)
        {
            u16_minSpeed = testParam.u32_Data;
        }
        
        TestParam.u8_PID = eU16_Current_mA;     // from tEN_SYSPARAM_ID
        if (UFu_GetDrvParam(&testParam)==eMS_OK)
        {
            u16_maxCurrentmA = testParam.u32_Data;
        }
        break;
    }
	

//###########################################
//#     Store nonvolatile user variables    #
//###########################################
	// Set current position to nv user variable
	UVar.UserNonVolatile.i32_UserVar[0] = UVar.i32_ActualPos_um_01deg;
	// Store the nv user variables
	UFu_StoreUserData();


//###########################################
//#            Homing on stall              #
//###########################################
    case HOMING_ON_STALL:
    {
        // Homing until stall, torque 100%, timeout 10s and speed/direction -20rpm
        stAppCSPS.pHomingCntrl->enMode = Homing_tilSTALL | 0x0F; // Change to homing mode, with torque 0x0F (100%)
        stAppCSPS.pHomingCntrl->u16_Timeout_ms = 10000;                		
        if (UFu_HomingPrepFuncP(-20) == eMS_OK)                  // Call homing function with -20rpm
        {
            UVar.u8_StepChain[0]++;
        }
        break;
    }


//###########################################
//#            Homing on input              #
//###########################################
// Homing_DIx_gLo_sm --> Homing Move until Input<n> goes LOW or Timeout,  single Move
// Homing_DIx_gHi_sm --> Homing Move until Input<n> goes HIGH or Timeout, single Move
// Homing_DIx_gLo_dm --> Homing Move until Input<n> goes LOW or Timeout,  double Move
// Homing_DIx_gHi_dm --> Homing Move until Input<n> goes HIGH or Timeout, double Move

    case HOMING_ON_INPUT:
    {
        // Homing until Din1, timeout 10s and speed/direction -20rpm
        stAppCSPS.pHomingCntrl->enMode = Homing_DIx_gHi_sm | 0x01; // Homing until Din1 goes HIGH or timeout, single move
        stAppCSPS.pHomingCntrl->u16_Timeout_ms = 10000;                		
        if (UFu_HomingPrepFuncP(-20) == eMS_OK)                    // Call homing function with -20rpm
        {
            UVar.u8_StepChain[0]++;
        }
        break;
    }


//###########################################
//#            Set 0 position               #
//###########################################
    case SET_0_POS:
    {
        // Check if motor HOLD or IDLE
        if (UVar.enDrvState <= eMS_HOLD)
        {
            // Set 0 position
            stAppCSPS.pHomingCntrl->enMode = Homing_StoreHome; // Use "Homing_StoreMarkasHome" when homing on stall for better precision
            if (UFu_HomingPrepFuncP(0)==eMS_OK)                // Call Homing Function
            {
                UVar.u16_Timer5ms[0] = 200;	// Set delay of 200*5ms=1s
                UVar.u8_StepChain[0]++;
            }
        }
        break;
    }


//###########################################
//#           Set custom position           #
//###########################################
    case SET_CUSTOM_POS:
    {
        // Set custom position to 5000
        if (UFu_SetPosition(5000, eGOTO_um_01deg) == eMS_OK) // value in [um] or [0.1°]
        //if (UFu_SetPosition(5000, eGOTO_ustps) == eMS_OK)  // value in [usteps]
        {
            UVar.u8_StepChain[0]++;
        }
        break;
    }
	

//###########################################
//#      Go to position with tolarance      #
//###########################################
#define i32_next_pos    UVar.i32_PosVAR_um_01deg[0]    // Next position
#define i32_calc_var    UVar.i32_PosVAR_um_01deg[1]    // Calculation variable
#define i32_act_pos     UVar.i32_ActualPos_um_01deg    // Actual position

    case GOTOPOS:
    {
        // Calc diff from wanted position to actual position
        i32_calc_var = i32_next_pos - i32_act_pos;
        
        // Diff as absolute value
        if(i32_calc_var < 0)
        {
            i32_calc_var *= -1;
        }
        // If position outside tolerance, call go to function
        if(i32_calc_var > 20)
        {
            if(UFu_GotoFuncP(i32_next_pos, eGOTO_um_01deg) == eMS_OK)
            {
            }
        }
        else
        {
            // Next case
            UVar.u8_StepChain[0]++;
        }
        break;
    }


//###########################################
//#                Rotate                   #
//###########################################
    case ROTATE:
    {
        // Call rotate function with 120rpm (in 0.1rpm)
        if(UFu_RotateFuncP(1200) == eMS_OK)
        {
            UVar.u8_StepChain[0]++;
        }
        break;
    }


//###########################################
//#               Fast Stop                 #
//###########################################
//# For emergency stop, no ramp             #
//# Use UFu_RotateFuncP(0) when ever        #
//# possible                                #
//###########################################
    case FAST_STOP:
    {
        // Do fast stop
        UFu_DrvFastStop;
    }
	

//###########################################
//#        Set/Reset digital outputs        #
//###########################################
    // Set digital output 1
    UVar.u8_Dout.ucAllBits |= SET_DO1_PNP_MSK;
    UVar.u8_Dout.ucAllBits &= ~SET_DO1_NPN_MSK;
    
    // Reset digital output 2
    UVar.u8_Dout.ucAllBits &= ~SET_DO2_PNP_MSK;
    UVar.u8_Dout.ucAllBits |= SET_DO2_NPN_MSK;


//###########################################
//#         Check for digital inputs        #
//###########################################
    // Check if Din1 is HIGH
    if ((UVar.u8_Din.ucAllBits & 0x02) == 0x02)					
    {
        // Do something, or go to case
    }
    // ...or
    if (UVar.u8_Din.b.bDI1)
    {
        // Do something, or go to case
    }
    
    // Check if Din1 is LOW
    if ((UVar.u8_Din.ucAllBits & 0x02) == 0x00)
    {
        // Do something, or go to case
    }
    // ...or
    if (!UVar.u8_Din.b.bDI1)
    {
        // Do something, or go to case
    }
    
    // Check if Din1 is RISING
    if (((UVar.u8_Din.ucAllBits & 0x02) == 0x02) && (UVar.u8_DinChange.ucAllBits & 0x02))
    {
        // Do something, or go to case
    }
    
    // Check if Din1 is FALLING
    if (((UVar.u8_Din.ucAllBits & 0x02) == 0x00) && (UVar.u8_DinChange.ucAllBits & 0x02))
    {
        // Do something, or go to case
    }


//###########################################
//#          Set/Reset user LED             #
//###########################################
    // Set/Reset user LED directly
    UFu_SetDigitalOuts(0x00);                  // User LED all OFF
    UFu_SetDigitalOuts(SET_LEDred_MSK);        // User LED red ON
    UFu_SetDigitalOuts(SET_LEDgreen_MSK);      // User LED green ON
    
    // Alternatively, is set in normal cycle
    UVar.u8_Dout.ucAllBits = SET_LEDgreen_MSK; // User LED green ON
    UVar.u8_Dout.bK11ab.bUserLEDred = 0;       // User LED red OFF
    UVar.u8_Dout.bK11ab.bUserLEDgreen = 1;     // User LED green ON
	
	
//###############################################################
//#                     Analog In/Out                           #
//###############################################################
//#                                                             #
//#    // Ain of different controller                           #
//#    UVar.KxxFlexUser.K17a.u16_Ain0to24_8d86mVpE    // K17a   #
//#    UVar.KxxFlexUser.K17c.u16_Ain0to24_8d86mVpE    // K17c   #
//#    UVar.KxxFlexUser.K17e.u16_Ain0to24_8d86mVpE    // K17e   #
//#    UVar.KxxFlexUser.K17f.u16_Ain4to20mA_0d01mApE  // K17f   #
//#    UVar.KxxFlexUser.K17g.u16_Ain0to10v_1mVpE      // K17g   #
//#    UVar.KxxFlexUser.K11a.u16_Ain4to20mA_0d01mApE  // K11a   #
//#    UVar.KxxFlexUser.K11b.u16_Ain0to10v_1mVpE      // K11b   #
//#    UVar.KxxFlexUser.D17a.u16_Ain0to10v_1mVpE      // D17a   #
//#                                                             #         
//#    // Aout of different controller                          #         
//#    UVar.KxxFlexUser.K11a.u16_Aout4to20mA_0d01mApE // K11a   #
//#    UVar.KxxFlexUser.K11b.u16_Aout0to10V_1mVpE     // K11b   #
//#                                                             #
//###############################################################
    // Example route Ain to Aout for K11a controller (4-20mA)
    UVar.KxxFlexUser.K11a.u16_Aout4to20mA_0d01mApE = UVar.KxxFlexUser.K11a.u16_Ain4to20mA_0d01mApE;
    // Example route Ain to Aout for K11b controller (0-10V)
    UVar.KxxFlexUser.K11b.u16_Aout0to10V_1mVpE = UVar.KxxFlexUser.K11b.u16_Ain0to10v_1mVpE;


//###########################################
//#            Serial Rx event              #
//###########################################
// ComData: Use either 'pComData->' OR 'stAppCSPS.SPSUserVar.st32_ComData.'
//          with u8_CMD, i8_CMD (Byte 1), u8_Data, i8_Data (Byte 2) u16_Data, i16_Data (Byte 3&4), u32_data, i32_data (all)
// Example: Go to pre defined position 1 or 2 (cmd byte0) with desired run torque (data byte1).
//          Sent over serial interface with 'Df' command, 4 databytes (stAppCSPS.SPSUserVar.st32_ComData.u8_CMD, .u8_Data, .u16_Data)
//          <Df#0D#XL1[0x01]#XL1[50]#XL2[0]#CK> --> 0x44 0x66 0x0D 0x01 0x32 0x00 0x00 0x16
    #define POSITION_1	3600
    #define POSITION_2	7200
    
    void LIFunc_WEAK AppCSPS_USER_SEQ_232_RX_Event(tCSPSCOMRXDATA* pComData)	//!< RW: from Serial Port received data Command <Df>, shall be cleared by USER !)
    {
        switch(UVar.st32_ComData.u8_CMD)
        // switch(pComData->u8_CMD)
        {
            case 0x01:
                
                // Set torque RUN
                UFu_DrvChgSetting(16, UVar.st32_ComData.u8_Data);
                // Go to 360°
                if(UFu_GotoFuncP(POSITION_1, eGOTO_um_01deg) == eMS_OK)
                {
                    // Clear Data, so Rx event is handled
                    UVar.st32_ComData.u32_data = 0;
                }
                break;
            }
            case 0x02:
            {
                // Set torque RUN
                UFu_DrvChgSetting(16, UVar.st32_ComData.u8_Data);
                // Go to 720°
                if(UFu_GotoFuncP(POSITION_2, eGOTO_um_01deg) == eMS_OK)
                {
                    // Clear Data, so Rx event is handled
                    UVar.st32_ComData.u32_data = 0;
                }
                break;
            }
            default:    // Not known cmd
            {
                // Clear Data, so Rx event is handled (use either)
                UVar.st32_ComData.u32_data = 0;
                // pComData->u32_data = 0;
                break;
            }
        }
    }


//###########################################
//#          Serial Tx function             #
//###########################################
	// Send current position
	UFu_SendComUserMsg(UVar.i32_ActualPos_um_01deg);


//###########################################
//#            CAN Rx event                 #
//###########################################
// Example: React to CAN RPDO, rotate or stop
    /************************************************************************************************/
    /*!
     *  \brief    React to CANopen PDO4-Rx-Events.
     *  \details  Cmds: 1 - rotate 120rpm, 2 - stop
     *            Id     DLC   Byte0(cmd)	Byte1(not used)	Byte2(not used)	Byte3(not used)
     *            0x503	 4     0x01/0x02    0x00            0x00            0x00
     *
     ************************************************************************************************/
    void AppCSPS_USER_SEQ_CAN_RX_Event(tCANDATA* pRxData, UI_8 Datalength)
    {
        // .. My Command interpreter Example
        switch (pRxData->u08_Data[0])    // switch Byte0 (cmd)
        {
            // CMD=1: Rotate with 120rpm
            case 01:
            {
                UFu_RotateFuncP(1200);
                break;
            }
            // CMD=2: Stop rotating
            case 02:
            {
                UFu_RotateFuncP(0);
                break;
            }
        }
    }


//###########################################
//#             CAN Tx function             #
//###########################################
// Example: PDO4-Tx with 8 databytes (4 or 6 would also be possible)
// Id       DLC     Byte0   Byte1   Byte2   Byte3   Byte4   Byte5   Byte6   Byte7
// 0x503    8       0xF0    0xDE    0xBC    0x9A    0x78    0x56    0x34    0x12

    void AppCSPS_USER_SEQ_CAN_TX_SynxEvent(void)
    {
        tCANDATA CanTxData;
        CanTxData.u08_Data[0]=0xF0;
        CanTxData.i08_Data[1]=0xDE;
        CanTxData.u16_Data[1]=0x9ABC;
        CanTxData.i32_Data[1]=0x12345678;
        stAppCSPS.SPSCallFunctions.CANopen_PDO4_Send(&CanTxData,8);
    }


//###########################################
//#   Allow/block external move commands    #
//###########################################
//# By default, motion commands are blocked #
//# when the user sps (sequence) is active. #
//# This function allows enabling or        #
//# disabling this feature as needed.       #
//###########################################
	UFu_COM_MoveAllow(true);


//###########################################
//#  Multiple infos (calls/read/variables)  #
//###########################################
UVar.i32_ActualPos_um_01deg     // Get Actual position
UVar.enDrvState                 // Get Motor state
UVar.u8_StepChain[]             // StepChain Var (4x)
UVar.u16_Timer5ms[]             // Default timer var (4x). Is decremented automatically every 5ms loop.
UVar.i32_PosVAR_um_01deg[]      // Pos var (3x)
UVar.KxxFlexUser.u08_UserVar[]  // KxxFlexUser, 20-128bytes (depending on Controller)
UVar.KxxFlexUser.i08_UserVar[]  // These are overlapping, means u08_UserVar[0] is i08_UserVar[0]
UVar.KxxFlexUser.u16_UserVar[]  // and u16_UserVar[0] is i16_UserVar[0] and so on...
UVar.KxxFlexUser.i16_UserVar[]
UVar.KxxFlexUser.u32_UserVar[]
UVar.KxxFlexUser.i32_UserVar[]

// Check if motor is not moving
// See tenMOTSTATES in user header file L0_KannMotDrvTypes.h for more states
if(UVar.enDrvState <= eMS_HOLD)
