/*************************************************************************************************/
/*! \file
\ingroup AD_LIB
\brief    Adlos KannMotion Typendefinitionen welche zudem fr die CSPS noch gebarcuht werden

- xxx

CREATED
\date     2.7.2019
\version  1.0.0.0
\author   Michael Zimmermann
\remarks  -

<hr>MODIFIED
\date    -
\version -
\author  -
\remarks -

COPYRIGHT  2019, ADLOS AG, FL-9496 Balzers<hr>
**************************************************************************************************/
#ifndef __L0_KANNMOTION_CSPS_EXTRA_TYPES_h
#define __L0_KANNMOTION_CSPS_EXTRA_TYPES_h

// --------- Globale Typendefinitionen --------------------------

//! Enumeration State Machine
typedef enum
{
  App_INIT_WaitSupplyOK = 0,                                           //!< Warten bis Speisung ok ist, verhindert dauernden Error und damit StoragePos bei ung. Speisung
  App_INIT_Encoder  = 1,                                               //!< Initialisierung des Encoders, Daten Abfragen usw..
  App_INIT_DRV      = 2,                                               //!< Initialisierung des Drivers L6470
  App_Test          = 0xF,                                             //!< Testmodus, frs Prfssystem
  App_RUN      = 0x10,                                                 //!< Applikation, ausfhren
  App_RUN_SPS    = 0x15,                                               //!< SPS ausfhren
  App_HomingInit = 0x20,                                               //!< Homing Initialisieren-Starten
  App_HomingMov1 = 0x21,                                               //!< Homing Movement 1
  App_HomingDelay= 0x22,                                               //!< Homing Delay
  App_HomingMov2 = 0x23,                                               //!< Homing Movement 2
  App_Error = 0x80,                                                    //!< Applikation im Fehler
}
tenAPPSTATES;

//!< Fehler BitStruktur
typedef union DO_ATTR_PACK
{
  UI_16 u16AllErrors;                                                     //!< R, all Errors together
  struct DO_ATTR_PACK
  {
    BIT(bDrvOverCurrent);                                                 //!< R: Driver zeigt berstrom
    BIT(bDrvOverTemp);                                                    //!< R: PowerStep oder 6470 zeigt bertemperatur-Fehler
    BIT(bUinToLo);                                                        //!< R: Eingangsspannung zu tief
    BIT(bTimeout);                                                        //!< R: State Timeout, z.B. Homing
    BIT(bStepDrvInit);                                                    //!< R: Stepdriver Init Fehlgeschlagen
    BIT(bEncoderInit);                                                    //!< R: Encoder Init Fehlgeschlagen
    BIT(bUmotToLo);                                                       //!< R: Eingangsspannung der Motorenseite zu tief
    BIT(bEncOverTemp);                                                    //!< R: Sensor zeigt bertemperatur-Fehler

    BIT(bStall);                                                          //!< R: blockade wurde festgestellt
  }b;
}
tERRBITS;
// Fehlermasken
#define u16ERR_MSK_RESET_ERRORS      0x0039                               //!< diese Fehler drfen automatisch (nach Zeit) zurckgesetzt werden
#define u16ERR_MSK_TIMEOUT_ERRORS    0x0138                               //!< diese Fehler gehen in die Timeout & STALL Statistik
#define u16ERR_MSK_TEMP_ERRORS       0x0081                               //!< Thermische Fehler

//! Steuerstruktur NEMA Digital Outputs 1-2
typedef union DO_ATTR_PACK
{
  UI_8 ucAllBits;                                                       //!< R:   alle Control& Info Bits
  struct DO_ATTR_PACK
  {
      BIT(bO1_NPN);                                                   //!< Zustand O1-NPN
      BIT(bO1_PNP);                                                   //!< Zustand O1-PNP
      BIT(bO2_NPN);                                                   //!< Zustand O2-NPN
      BIT(bO2_PNP);                                                   //!< Zustand O2-PNP
  }b;
}
unDOUTS12t;
#define SET_DO1_NPN_MSK  0x1
#define SET_DO1_PNP_MSK  0x2
#define SET_DO2_NPN_MSK  0x4
#define SET_DO2_PNP_MSK  0x8

//! Infostruktur Inputs/Drive Status
typedef union DO_ATTR_PACK
{
  UI_8 ucAllBits;                                                       //!< R:   alle Control& Info Bits
  struct DO_ATTR_PACK
  {
      BIT(bDI0);                                                   //!< Zustand DIN(0) Ain
      BIT(bDI1);                                                   //!< Zustand DIN(1)
      BIT(bDI2);                                                   //!< Zustand DIN(2)
      BIT(bDI3);                                                   //!< Zustand DIN(3)
      BIT(bDI4);                                                   //!< Zustand DIN(3)
      BIT(bNC);
      BIT(bDrv_nFlag);                                             //!< Zustand von L6470 oder Powerstep FlagPin, zeigt 1 wenn Low da Lowaktiv
      BIT(bDrv_nBusy);                                             //!< Zustand von L6470 oder Powerstep BusyPin, zeigt 1 wenn Low da Lowaktiv
  }b;
}
unDIN_Infot;
#define DIN_INFO_DI4_BITS_MSK        0x10                          //!< DI4 maskiert
#define DIN_INFO_CLR_DRV_BITS_MSK    0x3F                          //!< Lschmaske

//! Enumeration Homing Modes
typedef enum
{
  Homing_StoreHome  = 0x00,                                            //!< Save actual Position as Home
  Homing_tilSTALL   = 0x10,                                            //!< Homing Move until Stall or Timeout
  Homing_DIx_gLo_sm = 0x20,                                            //!< Homing Move until Input<n> goesLo or Timeout, single Move
  Homing_DIx_gHi_sm = 0x30,                                            //!< Homing Move until Input<n> goesHi or Timeout, single Move
  Homing_DIx_gLo_dm = 0x40,                                            //!< Homing Move until Input<n> goesLo or Timeout, double Move
  Homing_DIx_gHi_dm = 0x50,                                            //!< Homing Move until Input<n> goesHi or Timeout, double Move
}
tenHOMINGMODES;
#define HOMING_MODE_MASK 0xF0                                          //!< Maske fr die oberen 4-Bit

//! Homing CNTRL Struktur
typedef struct  DO_ATTR_PACK
{
  tenHOMINGMODES  enMode;                                                 //!< RW: Modus
  UI_16           u16_Timeout_ms;                                         //!< RW: Homing Timeout in [ms]
  SI_16           i16_Speed_01rpm;                                        //!< R:  Homing Speed in [0.1rpm]
  unDIN_Infot     u8_DinMask;                                             //!< R:  selektierter Eingang
}
tHOMINGCNTRL;

#endif // __L0_KANNMOTION_CSPS_EXTRA_TYPES_h
