/*************************************************************************************************/
/*! \file
\ingroup AD_LIB
\brief    Adlos KannMotion MotorDriver Types (allgemeine Typendefinitionen)

- xxx

CREATED
\date     28.6.2019
\version  1.0.0.0
\author   Michael Zimmermann
\remarks  -

<hr>MODIFIED
\date    6.12.2019
\version 1.0.0.1
\author  Michael Zimmermann
\remarks Parameter fr DC-Motor integriert

COPYRIGHT  2019, ADLOS AG, FL-9496 Balzers<hr>
**************************************************************************************************/
#ifndef __L0_KANNMOTION_DRV_TYPES_h
#define __L0_KANNMOTION_DRV_TYPES_h

// --------- Globale Typendefinitionen --------------------------
//! Microsteps
typedef enum
{
  eMS_Full = 0,                                                    //!< Full Step
  eMS_Half  = 1,                                                   //!< Half
  eMS_Quarter = 2,                                                 //!< Quarter
  eMS_08uSt = 3,                                                   //!<  8-tel  Schritt
  eMS_16uSt = 4,                                                   //!< 16-tel  Schritt
  eMS_32uSt = 5,                                                   //!< 32-tel  Schritt
  eMS_64uSt = 6,                                                   //!< 64-tel  Schritt
  eMS_128uSt = 7,                                                  //!< 128-tel Schritt
  eMS_4096uSt = 12,                                                //!< 4096-tel Schritt
}
tenMICROSTEPS;

//! Motoren, Driver Zustnde
typedef enum
{
  eMS_RESET  = 0x0,                                                //!< ist im Reset
  eMS_IDLE = 1,                                                    //!< Motor-Treiber ist in Standby / Sleep / .. Kraftlos
  eMS_HOLD = 2,                                                    //!< Motor-Treiber ist ein, Motor steht
  eMS_RELEASEDELAY = 3,                                            //!< mssen noch kurz den Stop auskosten
  eMS_ROTATE = 0x10,                                               //!< fhren Rotate AUS
  eMS_GOTOPOS = 0x20,                                              //!< fahren Position an
  eMS_DoSTOP  = 0x80,                                              //!< mache Stop
}
tenMOTSTATES;

//! Fehlercodes
typedef enum
{
  eMS_OK = 0,                                                      //!< kein Fehler
  eMS_ERR_OUTofRange = -1,                                         //!< Fehler, Parameter ausserhalb gltigem Bereich
  eMS_ERR_ParamisWrProtected = -2,                                 //!< Fehler, Parameter kann nicht geschrieben werden
  eMS_ERR_CMDnotAccepted = -3,                                     //!< Fehler, Kommando konnte nicht ausgefhrt werden, da z.B. noch in Arbeit
  eMS_ERR_CMDnotKnown = -4,                                        //!< Fehler, Kommando nicht bekannt
  eMS_ERR_ParamisNotKnown = -5,                                    //!< Fehler, Parameter ist unbekannt
  eMS_ERR_ActionFailed = -6,                                       //!< Fehler, Aktion Fehlgeschlagen
  eMS_ERR_NoMoveUntilRestart = -7,                                 //!< Fehler, Bewegung nicht erlaubt bis Neustart/Init
}
tenMOTERRCODE;

//! Write Protection Levels
typedef enum
{
   PAR_WRPROT_NORM = 0,                                            //!< Normal Level, d.h. geschtze Stellen knnen nicht geschriben werden
   PAR_WRPROT_RWP  = 0x10,                                         //!< RWP    Parameter drfen geschrieben werden
   PAR_WRPROT_RO   = 0x30,                                         //!< RO+RWP Parameter drfen geschrieben werden (hoch)
}
tenWRP_LEVELS;

//! Motorenwinkel / Schritt  -> Anzahl Schritte/ Halbe Umdrehung
typedef enum
{
  eMS_009 = 200,                                                   //!<  0.9 -> 400 Schritte/Umdrehung
  eMS_018 = 100,                                                   //!<  1.8 -> 200 Schritte/Umdrehung
  eMS_075 =  24,                                                   //!<  7.5 ->  48 Schritte/Umdrehung
  eMS_150 =  12,                                                   //!< 15.0 ->  24 Schritte/Umdrehung
  eMS_1DC =  0,                                                    //!<  DC Motor
}
tenMOTSTEPTYPE;

//! Event Flags
typedef union DO_ATTR_PACK
{
  UI_8 ucAll;                      //!< RW: alle Bits zusammen
  struct DO_ATTR_PACK
  {
    BIT(bStatRefrClearWarn);       //!< RW : Status Referesh mit ErrWarnClear
    BIT(b1);
    BIT(b2);
    BIT(bRegPulse);                //!< R&C: Nachregelungs Event Flag
    BIT(bMStopEventApp);           //!< R&C: zeigt dass Bewegung beendet ist
    BIT(bMStopEventCANOpen);       //!< R&C: zeigt dass Bewegung beendet ist	  
  };
}
tMOTEVENTFLAGS;
#define MOT_STOPEVENTFLAGMASK 0xF0;

//! Factory Driver Configuration Bits
typedef union DO_ATTR_PACK
{
  UI_8 ucAll;                      //!< RW: alle Bits zusammen
  struct DO_ATTR_PACK
  {
    BIT(bMotMoveDir_Invers);             //!< R: Motor soll invertiert gesteuert werden (verkehrter Motor)
    BIT(bEncoderPos_onGearAxis);         //!< R: Encoder ist auf Getriebe montiert
    BIT(bTakeStoredPosEvenifRotorNotOK); //!< R: es wird die gespeicherte Position genommen, auch wenn Rotor nicht mehr dort steht !
  };
}
tFACDRVCONF;

//! Hilfsstruktur fr Steuerdaten-Zerlegung
typedef struct
{
  bool   bDirClockwise;      //!< R: Drehrichtung
  UI_32  lAbsValue;          //!< R: Wert ohne Vorzeichen
}
ST_MOT_CMD_DATA;

//! Zielwerte
typedef struct
{
  UI_32  uiSpeed_stps;             //!< R: [Steps/s] oder [Hz] Ziel-Speed
  SI_32  iPosition;                //!< R: Position in Steps resp. je nach Einstellung Quater usw..
}
tMOTDRV_TARGET;

//! Info-Bits
typedef union DO_ATTR_PACK
{
  UI_8 ucAll;                      //!< R: alle Bits zusammen
  struct DO_ATTR_PACK
  {
    BIT(bNewStartNeeded);          //!< R: Parameter-nderung verlangt Neustart-Init vom System, Bewegungen sind nun verboten
  };
}
tMOTDRV_INFOLAGS;
//! Aktuelle Statusdaten fr andere Module
typedef struct DO_ATTR_PACK
{
  tenMOTSTATES enState;            //!< R: aktueller Zustand
  tMOTDRV_INFOLAGS u8InfoBits;     //!< R: InfoBits
  UI_16  u16_StepsPerRound;        //!< R: gerechnete Anzahl Schritte pro Runde
  SI_32  i32_Position;             //!< R: Position in Steps resp. je nach Einstellung Quater usw..
}
tMOTDRV_ACTUAL;

//! KannMOTION CONTROL_Mode
typedef enum
{
  MODE_um = 0x00,                                                       //!<  Steuermode in MikroMeter
  MODE_01g= 0x01,                                                       //!<  Steuermode in 1/10
}
tenCNTRL_MODE;

//! KannMOTION Standard Output Configuration
typedef enum
{
  DOx_OFF = 0x00,                           //!<  Ausgang deaktiviert
  DOx_NPN = 0x01,                           //!<  Ausgang in NPN Mode
  DOx_PNP = 0x02,                           //!<  Ausgang in PNP Mode
  DOx_PP  = 0x03,                           //!<  Ausgang in Push/Pull ... NPN&PNP sind ein
}
tenOUTCONFIG;

//! KannMOTION Digital Outputs 1-2
typedef union DO_ATTR_PACK
{
  UI_8 ucAllBits;                                                     //!< R:   alle Control& Info Bits
  struct DO_ATTR_PACK
  {
      BIT(bO1_NPN);                                                   //!< Zustand O1-NPN
      BIT(bO1_PNP);                                                   //!< Zustand O1-PNP
      BIT(bO2_NPN);                                                   //!< Zustand O2-NPN
      BIT(bO2_PNP);                                                   //!< Zustand O2-PNP
  }b;
}
tOUTCONFIG;
#define OUT_DO1_NPN_MSK  0x1                  //!<  NPN ansteuern
#define OUT_DO1_PNP_MSK  0x2                  //!<  PNP ansteuern
#define OUT_DO1_PP_MSK   0x3                  //!<  PP: NPN & PNP ansteuern
#define OUT_DO2_NPN_MSK  0x4                  //!<  NPN ansteuern
#define OUT_DO2_PNP_MSK  0x8                  //!<  PNP ansteuern
#define OUT_DO2_PP_MSK   0xC                  //!<  PP: NPN & PNP ansteuern

//! Torque Setting Structure, different values dep.on state
typedef union DO_ATTR_PACK
{
  UI_32  All4_Torques;                        //!< RW: RUN, HOLD, ACC, DEC Torque  in [0.5%/E]
  struct DO_ATTR_PACK
  {
    UI_8 Hold_hPct;                           //!< RW: Hold Torque in [0.5%/E]
    UI_8 ACC_hPct;                            //!< RW: ACC Torque in [0.5%/E]
    UI_8 Run_hPct;                            //!< RW: RUN Torque in [0.5%/E]
    UI_8 DEC_hPct;                            //!< RW: DEC Torque in [0.5%/E]
  };
}
tDRV_TORQ_PERC;

//! KannMOTION Regulator Control Setting
typedef union DO_ATTR_PACK
{
  UI_8 ucAllBits;                                                     //!< R: alle Control Bits
  struct DO_ATTR_PACK
  {
      BIT(b0Tolerance);                                               //!< Toleranzwert Bit0 1/4 Step
      BIT(b1Tolerance);                                               //!< Toleranzwert Bit1 1/4 Step
      BIT(b2Tolerance);                                               //!< Toleranzwert Bit2 1/4 Step
      BIT(b3Tolerance);                                               //!< Toleranzwert Bit3 1/4 Step
      BIT(b4_NC);                                                     //!< Zustand O1-PNP
      BIT(b5_NC);                                                     //!< Zustand O2-NPN
      BIT(b6_NC);                                                     //!< Zustand O2-PNP
      BIT(b7_RegEnable);                                              //!< Positionsregler ENABLE
  }b;
}
tREGCONTRL;
#define REG_CNTRL_TOL_MSK  0x0F               //!<  Regler CNTRL- Toleranz Wert- Maske

//! KannMOTION Input ( Analog und Digital ) Filter Control Byte
typedef union DO_ATTR_PACK
{
  UI_8 ucAllBits;                                                      //!< R:   alle Control& Info Bits
  struct DO_ATTR_PACK
  {
      BIT(bDI0_deBounceON);                                            //!< DIN(0) zustzlicher Debounce Filter <EIN>
      BIT(bDI1_deBounceON);                                            //!< DIN(1) zustzlicher Debounce Filter <EIN>
      BIT(bDI2_deBounceON);                                            //!< DIN(2) zustzlicher Debounce Filter <EIN>
      BIT(bDI3_deBounceON);                                            //!< DIN(3) zustzlicher Debounce Filter <EIN>
      BIT(bDI4_deBounceON);                                            //!< DIN(4) zustzlicher Debounce Filter <EIN>
      BIT(bNC);
      BIT(bAI0_FIL0);                                                  //!< Filter Selection for Analog input0 Bit0
      BIT(bAI0_FIL1);                                                  //!< Filter Selection for Analog input0 Bit1
  }b;
}
tINPUTFIL_CNTRL;
#define ADIN_AI0FIL_MSK              0xC0                              //!< AIN0-Filter Selection Mask
#define ADIN_FIL_MEDIAN              0x80                              //!< Analog Median Filter
#define ADIN_FIL_MEAN                0x40                              //!< Analog MEAN   Filter
#define ADIN_FIL_RES                 0xC0                              //!< Analog noch reservierter Filter
#define ADIN_DINFIL_MSK              0x1F                              //!< DINx-Filter Selection Mask

//! KannMotion Driver Parameter Einzeln
typedef struct DO_ATTR_PACK
{
  UI_8           u8_PID;                      //!< R: Parameter-ID
  UI_8           u8_ModeSize;                 //!< R: Modus & Grsse
  UI_32          u32_Data;                    //!< R: Inhalt
}
tMOTDRV_PARAM;

//! SteuerModes fr GotoPos
typedef enum
{
  eGOTO_ustps = 0,                                                 //!< Goto Positon, Value in MicroSteps
  eGOTO_um_01deg = 1,                                              //!< Goto Positon, Value in Micrometer or 0.1 depending on Drive Configuration
  eGOTO_ustps_Relative = 2,                                        //!< Goto Positon, Value in MicroSteps, drive CNT Steps Releative to actual Pos
}
tenGOTOPOSMODES;



#endif // __L0_KANNMOTION_DRV_TYPES_h
