/*************************************************************************************************/
/*! \file
\ingroup AD_LIB
\brief   this Header File defines the ADLOS / TRIDONIC.ATCO / TEFAG Standard Datatypes

written for ARM-GCC COMPILER for STM32 Micros
This File has to be replaced if you change to another compiler

die Orientierung der Strukturen und des Compilers ist 'LIDDLE ENDIAN' !
siehe auch http://de.wikipedia.org/wiki/Byte-Reihenfolge

CREATED
\date     26.6.2019
\version  1.0
\author   Michael Zimmermann, ADLOS
\remarks  -

<hr>MODIFIED
\date     27.11.2019
\version  1.0
\author   Michael Zimmermann, ADLOS
\remarks  kleinere Anpassungen, inkl. allgmeinen Funktionspointer

COPYRIGHT  2019, ADLOS AG, 9496 Balzers<hr>
**************************************************************************************************/
#ifndef TADATATYPES_H
#define TADATATYPES_H

  // --------- COMPILER INCLUDES, according ATMEL AVRGCC --------------
  #include <stdint.h>               // Standard Integer Types
  #include <stdbool.h>

   // ******* special Compiler/ Linker directives / normaly used for >8Bit CPUs
   #define DO_ATTR_PACK       __attribute__((__packed__))                    //!< Structure needs to be packed
   //#define DO_PACK            __packed                                     //!< Structure -Var needs to be packed, bei Keil zeigt das, dass der Compiler auf unausgerichtete Vars trefefn kann
   #define DO_ALLIGN4          __attribute__ ((aligned(4)))                  //!< setzen damit Start zB auf gerader Adresse liegt (Teilpadding Compiler zulassen)
   #define DO_ALLIGN1          __attribute__ ((aligned(1)))                  //!< setzen um Compiler klar zu machen, dass es nicht auf 32Bit gepaded ist

   #define DO_PACK_ALLIGN4     __attribute__((packed,aligned(4)))

//assume_aligned
   //#define             __packed                                       //!< Structure -Var needs to be packed
   //#define DO_PADDING         __padding                                      //!< Structure -Var needs to be padded

   // ******* Unsigned Integer Types, abgeleitet vom C99-Standard ******************************
   typedef uint8_t   UI_8;                                                   //!<  8 Bit -> [0..255]
   typedef uint16_t UI_16;                                                   //!< 16 Bit -> [0..65535]
   typedef uint32_t UI_32;                                                   //!< 32 Bit -> [0..4'294'967'295]
   typedef uint64_t UI_64;                                                   //!< 64 Bit

   // ******* signed Integer Types ******************************
   typedef int8_t   SI_8;                                                    //!<  8 Bit -> [ -128  .. +127]
   typedef int16_t SI_16;                                                    //!< 16 Bit -> [-32768 .. +32767]
   typedef int32_t SI_32;                                                    //!< 32 Bit -> [-2147483648 .. +2147483647]
   typedef int64_t SI_64;                                                    //!< 64 Bit

   // ******* floating point types  ******************************
  // typedef float F_24;                                                     //!< 24 Bit  real
   typedef float F_32;                                                       //!< 32 Bit  real
   typedef double F_64;                                                      //!< 64 Bit  real

   // ******* Special Definitions    ******************************
   #define BIT(var) 	  UI_8 var:1                                           //!<  1 Bit -> use it only for Bitfields
   #define NIBBLE(var)  UI_8 var:4                                           //!<  4 Bit -> use it only for Bitfields

   // ******* Speziell fr Pointeraktionen mit 32Bit CPU.... packed und Alligned geben auskunft dass es eben nicht auf 4-Byte alligned ist
   //! UI_32 type, sicherer Zugriffsstyp fr Pointeraktionen mit 32-Bit CPU, ntig da GCC _packed im Typecast nicht akzeptiert
   typedef struct __attribute__((packed,aligned(1)))
   {
	 uint32_t u32Val;
   }
   tUI32Safe;
   //! UI_16 type, sicherer Zugriffsstyp fr Pointeraktionen mit 32-Bit CPU, ntig da GCC _packed im Typecast nicht akzeptiert
   typedef struct __attribute__((packed,aligned(1)))
   {
	 uint16_t u16Val;
   }
   tUI16Safe;
   //! UI_8 type, sicherer Zugriffsstyp fr Pointeraktionen mit 32-Bit CPU, ntig da GCC _packed im Typecast nicht akzeptiert
   typedef struct __attribute__((packed,aligned(1)))
   {
	 uint8_t u8Val;
   }
   tUI8Safe;

   // ******* Standard Predefined Unions ******************************
   //! One Byte, can be RW as Byte or as single Bits, may be alse a sample for your own spec. types
   typedef union
   {
     UI_8 ucByte;                                                            //!< all 8 Bits
     //! Single bits
     struct
     {
      BIT(b0);                                                              //!< Bit 0, if you do your own type, set clear names for each bit
      BIT(b1);                                                              //!< Bit 1
      BIT(b2);                                                              //!< Bit 2
      BIT(b3);                                                              //!< Bit 3
      BIT(b4);                                                              //!< Bit 4
      BIT(b5);                                                              //!< Bit 5
      BIT(b6);                                                              //!< Bit 6
      BIT(b7);                                                              //!< Bit 7
     }
     b;
   }
   UN_UI8;

   //! Word operation, can be operated as word (16-Bit) or in HiByte and LowByte (more eff. than shifting !! )
   typedef union DO_ATTR_PACK
   {
     UI_16 uiWord;                                                           //!< alle 8 Bits
     //! in bytes
     struct DO_ATTR_PACK
     {
       UI_8 ucLoByte;                                                        //! LoByte of Variable
       UI_8 ucHiByte;                                                        //! HiByte of Variable
     }
     by;
   }
   UN_UI16;

   //! DWORD Operation (32-Bit), also in splitted bytes possible
   typedef union DO_ATTR_PACK
   {
     UI_32 u32_Word;                                                         //!< alle 32 Bits zusammen
     //! in bytes
     struct DO_ATTR_PACK
     {
       UI_8 u8_Bytes[4];                                                     //!< alles Byteweise Bytey[0] ist LSB
     }
     by;
     //! in words
     struct DO_ATTR_PACK
     {
       UI_16 u16_Words[2];                                                   //!< alles Byteweise Bytey[0] ist LSB
     }
     w;
     //! in Mix
     struct DO_ATTR_PACK
     {
       UI_8  u8_LoByte;                                                      //!< Lo Byte
       UI_16 u16_MeWord;                                                     //!< Mittlere 16Bit
       UI_8  u8_HiByte;                                                      //!< Lo Byte
     }
     m;
   }
   UN_UI32;

   //!  Operation (64-Bit), also in splitted bytes possible
   typedef union DO_ATTR_PACK
   {
     UI_64 u64_Word;                                                         //!< alle 64 Bits zusammen
     UI_32 u32_Word[2];                                                      //!< 2x32 Bits
     UI_16 u16_Word[4];                                                      //!< 4x16 Bits
     UI_8  u8_Byte[8];                                                       //!< 8x08 Bits
   }
   UN_UI64;

   //! Standard function pointer
   typedef void (*tFuncP) (void);

   // ------------- Standard Makros , they are needed but syntax ist mostly compiler dependendent --------------
   // we define here they with Standard macro names, that software has not to be changed do to naming
//   #define  mCLRWDT() do { __asm__ __volatile__ ("wdr"); } while (0)         //!< Inserts a CLRWTD instruction, Library function
  // #define  mNOP()    __nop()                                        //!< Inserts a NOP instruction

   #define  mNOP()    do { __asm__ __volatile__ ("nop"); } while (0)         //!< Inserts a NOP instruction

//   #define  mSLEEP()  do { __asm__ __volatile__ ("sleep"); } while (0)       //!< Inserts a SLEEP instruction

   #define  mRESET()    NVIC_SystemReset()                                    //!< SoftReset
   //#define  mSLEEP    asm("SLEEP");                                         //!< Inserts a SLEEP instruction
   #define  mDI()     __disable_irq()                                         //!< Desable all interupts Set PRIMASK
   #define  mEI()     __enable_irq()                                          //!< Enable all interupts Clear PRIMASK
   #define  GetASPR() __get_APSR()                                            //!< Rckgabe UI_32, CPU Register APSR, unter anderem das Carry Flag! nur gekapselt zugreifen, wenn INTs gesperrt

// __WFE()  // Hlt CPU an

/*
WFI Sleep	__WFI()	Interrupt	Warte auf Interrupt. Nur die CPU wird angehalten.
WFE Sleep	__WFE()	Interrupt oder Ereignis	Warte auf Ereignis. Nur die CPU wird angehalten.
Stop	PDDS=1, LPDS=1, SLEEPDEEP=1 + __WFI() oder __WFE()	EXTI Leitung, configuriert im EXTI Register. 	Taktsignale von HSI/HSE sind deaktiviert. Die I/O Pins, Register und RAM bleiben unverndert. Debugging ist nicht mglich.
Standby	PDDS=1, LPDS=0, SLEEPDEEP=1 + __WFI() oder __WFE()	Steigende Flanke an WKUP Pin, RTC Alarm, externer Reset am NRST, IWDG Reset. 	Taktsignale von HSI/HSE sind deaktiviert. RAM Inhalte gehen verloren. Nur die Backup Register bleiben erhalten. Die I/O Pins werden hochohmig, auer TAMPER, WKUP und NRST. Debugging ist nicht mglich. Zum Aufwachen muss der Controller neu starten.
*/

//http://stefanfrings.de/stm32/stm32f3.html

//#define _CLI() do { __asm__ __volatile__ ("cli"); } while (0)
//#define _SEI() do { __asm__ __volatile__ ("sei"); } while (0)
//#define _NOP() do { __asm__ __volatile__ ("nop"); } while (0)
//#define _WDR() do { __asm__ __volatile__ ("wdr"); } while (0)





#endif   /*TADATATYPES_H_ */
