/*************************************************************************************************/
/*! \file
\ingroup Appl
\brief    Motor-APP / L2 SPS-Integration / USER Seite

- Integration kundenspezifischer CMDs
- bentigt spezielles Linker Skript

CREATED
\date     28.6.2019
\version  1.0
\author   Michael Zimmermann
\remarks  -

<hr>MODIFIED
\date    -
\version -
\author  -
\remarks -

COPYRIGHT  2019, ADLOS AG, FL-9496 Balzers<hr>
**************************************************************************************************/

#include "L0_TA_datatypes_GCC_ARM.h"                                      // uC Include & Std-TRIDONC Type - Definitions

#ifdef xOnlySPSpart
   #include "stm32f0xx.h"
   #include "L0_KannMotDrvTypes.h"
#else
  #include "L0_IncludeFiles.h"                                              // LowLevel Include Dateien
#endif

#include "L2_APPC_SPS_User.h"

// ---- Internal Definitions ----------------------------
//#define xDEBUG_INCLUDE_SPS_DEMOx1                                        // welche Demo soll kompiliert werden
#define xDEBUG_INCLUDE_SPS_DEMOxGEMU

// ---- SPS Control-Block ----------------------
volatile const LOCATEUSCNTRL  tAPPCSPS_CNTRLBLK   CNTRLBLKINIT = {
                                                                   .stCntrl.UserStandardFuncPtr_1= (tFuncP) 0xFFFFFFFF,
                                                                   .stCntrl.UserStandardFuncPtr_2= (tFuncP) 0xFFFFFFFF,
                                                                   .stCntrl.UserErrorFuncPtr= (tFuncP) 0xFFFFFFFF,
                                                                   .stCntrl.u32_BlkChkSum=0xFFFFFFFF,
                                                                    #ifdef fFeEXTRAFUNC
                                                                    #ifdef fFeCANRXTX
                                                                   .stCntrl.UserRxEventFuncPtr=AppCSPS_USER_SEQ_CAN_RX_Event,
                                                                    #endif
                                                                   .stCntrl.UserFuncReserved={
                                                                    (tFuncP) 0xFFFFFFFF,
                                                                    (tFuncP) 0xFFFFFFFF,
                                                                    (tFuncP) 0xFFFFFFFF,
                                                                    (tFuncP) 0xFFFFFFFF,
                                                                    (tFuncP) 0xFFFFFFFF,
                                                                    (tFuncP) 0xFFFFFFFF,
                                                                    (tFuncP) 0xFFFFFFFF },
                                                                    #endif
                                                                   .stInfo.su.u16_UserPrgVersion=0xFFFF,                     // Version
                                                                   .stInfo.su.u16_UserReserved=0xFFFF,                       // Version
                                                                   .stInfo.su.u8_UserPrgTxt={"---- NO ----"}                 // USER Text/ Beschreibung
                                                                 };
// ---- Globale Variabeln lsen -----------------
tAPPCSPS LOCATEVARSPS stAppCSPS;


/************************************************************************************************/
/*!
 *  \brief     Eigentlichen SPS-USER Funktionen // Leer Vorlage
 *  \details   unterteilt in 1&2 um allenfalls die Aufgaben zu splitten
 *  \author    Michael Zimmermann
 *  \version   1.0
 *  \date      28.6.2019
 *  \copyright ADLOS
*   \test
*   \param
*   \return
**************************************************************************************************/
#ifndef xDEBUG_INCLUDE_SPS_DEMOx1
#ifndef xDEBUG_INCLUDE_SPS_DEMOx2
#ifndef xDEBUG_INCLUDE_SPS_DEMOx3
#ifndef xDEBUG_INCLUDE_SPS_DEMOxGEMU
void LOCATEUSER AppCSPS_USER_SEQ_STANDARD_1(void)
{
	mNOP();
}
void LOCATEUSER AppCSPS_USER_SEQ_STANDARD_2(void)
{
	mNOP();
}
void LOCATEUSER AppCSPS_USER_SEQ_ERROR(void)
{
	mNOP();
}
#ifdef fFeCANRXTX
void LOCATEUSER AppCSPS_USER_SEQ_CAN_RX_Event(tCANDATA* pRxData, UI_8 Datalength)
{
}
#endif
#endif
#endif
#endif
#endif


#ifdef xDEBUG_INCLUDE_SPS_DEMOx1


/************************************************************************************************/
/*!
 *  \brief     SPS-USER Funktion / Demo <1>
 *  \details   Zeitfahrt mit 100rpm <2s>, danach GotoPos 100... endlos wiederholend
 *  \author    Michael Zimmermann
 *  \version   1.0
 *  \date      28.6.2019
 *  \copyright ADLOS
*   \test
*   \param
*   \return
**************************************************************************************************/
void LOCATEUSER AppCSPS_USER_SEQ_STANDARD_1(void)
{
  // -- Schrittvar ausgeproggt
  switch (stAppCSPS.SPSUserVar.u8_StepChain[0])
  {
    // Start, Rotate Befehl +100rpm, danach Delay Laden mit 2s, nchster Schritt
    case 0:
    {
      if (stAppCSPS.SPSCallFunctions.RotateFuncP(+1000)==eMS_OK)  // Call Rotate Function
      {
    	stAppCSPS.SPSUserVar.u8_Dout.ucAllBits = SET_DO1_PNP_MSK;   // Set Output 1
    	stAppCSPS.SPSUserVar.u16_Timer5ms[0]=400;                   // 2s -> 5ms x 400
        stAppCSPS.SPSUserVar.u8_StepChain[0]++;                   // next Step = Delay
      }
      break;
    }
    // Rotate Befehl 0 = Stop
    case 2:
    {
      if (stAppCSPS.SPSCallFunctions.RotateFuncP(0)==eMS_OK)
      {
        stAppCSPS.SPSUserVar.u8_StepChain[0]++;                 // next Step
      }
      break;
    }
    // Goto-Befehl zu Pos 100, danach Delay Laden mit 4s, nchster Schritt
    case 3:
    {
      if (stAppCSPS.SPSUserVar.enDrvState <= eMS_HOLD)                        // warten bis Stop ausgefhrt
      {
        if (stAppCSPS.SPSCallFunctions.GotoFuncP(100,eGOTO_um_01deg)==eMS_OK) // Call Goto Function
        {
    	  stAppCSPS.SPSUserVar.u8_Dout.ucAllBits = SET_DO2_PNP_MSK;  // Set Output 2
    	  stAppCSPS.SPSUserVar.u16_Timer5ms[0]=800;                  // 4s -> 5ms x 400
          stAppCSPS.SPSUserVar.u8_StepChain[0]++;                  // next Step = Delay
        }
      }
      break;
    }
    // neu von vorne
    case 5:
    {
      stAppCSPS.SPSUserVar.u8_StepChain[0]=0;
      break;
    }
    // Default Schritt ist quasi Delay Schritt, d.h. nach Ablauf wird Schrittzhler erhht
    default:
    {
      if (stAppCSPS.SPSUserVar.u16_Timer5ms[0]==0)            // wenn Timer bei Null, nchster Schritt
      {
        stAppCSPS.SPSUserVar.u8_StepChain[0]++;
      }
      break;
    }
  }//..switch end
}

void LOCATEUSER AppCSPS_USER_SEQ_STANDARD_2(void)
{
  mNOP();
  mNOP();
}

/************************************************************************************************/
/*!
 *  \brief     Eigentlichen SPS-USER Funktion ( Fehlerablauf ) // Leer Vorlage
 *  \details
 *  \author    Michael Zimmermann
 *  \version   1.0
 *  \date      28.6.2019
 *  \copyright ADLOS
*   \test
*   \param
*   \return
**************************************************************************************************/
void LOCATEUSER AppCSPS_USER_SEQ_ERROR(void)
{
  mNOP();
  mNOP();
  mNOP();
  mNOP();
}

#ifdef fFeCANRXTX
void LOCATEUSER AppCSPS_USER_SEQ_CAN_RX_Event(tCANDATA* pRxData, UI_8 Datalength)
{
}
#endif







#endif  //Demo1

#ifdef xDEBUG_INCLUDE_SPS_DEMOx2
/************************************************************************************************/
/*!
 *  \brief     SPS-USER Funktion / Demo <2>
 *  \details   Din-Muster <xxx0'011x> = Homing auf DIN3.....
 *             Din-Muster <xxx1'000x> = GotoPos 180
 *             Din-Muster <xxx0'100x> = GotoPos Pos+90
 *             Din-Muster <xxx0'001x> = GotoPos 0
 *             Merker[0] als Flankenmerker !
 *  \author    Michael Zimmermann
 *  \version   1.0
 *  \date      28.6.2019
 *  \copyright ADLOS
*   \test
*   \param
*   \return
**************************************************************************************************/
void LOCATEUSER AppCSPS_USER_SEQ_STANDARD_1(void)
{
  stAppCSPS.SPSUserVar.u8_Merker[0] ^= (stAppCSPS.SPSUserVar.u8_Din.ucAllBits & 0x1E);  // Flanken- nderungen feststellen


  // -- Schrittvar ausgeproggt
  switch (stAppCSPS.SPSUserVar.u8_StepChain[0])
  {
    // je nach Eingangskombination Zielposition zuweisen
    case 0:
    {
      // -- Zielposition gemss Schaltermuster zuweisen
      if ((stAppCSPS.SPSUserVar.u8_Din.ucAllBits & 0x1E) ==  0x10)
      {
    	stAppCSPS.SPSUserVar.i32_PosMerker[0]=1800;
      }
      else if ((stAppCSPS.SPSUserVar.u8_Din.ucAllBits & 0x1E) ==  0x02)
      {
    	stAppCSPS.SPSUserVar.i32_PosMerker[0]=0;
      }
      // -- bei Positiver Flanke, Gotopos CMD ausfhren lassen / Schrittmerker (+1)
      if (stAppCSPS.SPSUserVar.u8_Merker[0] & stAppCSPS.SPSUserVar.u8_Din.ucAllBits & 0x1E)
      {
        if ((stAppCSPS.SPSUserVar.u8_Din.ucAllBits & 0x1E) ==  0x08)      // +90
        {
          stAppCSPS.SPSUserVar.i32_PosMerker[0]+=900;
        }
        else if ((stAppCSPS.SPSUserVar.u8_Din.ucAllBits & 0x1E) ==  0x06) // Homing
        {
          stAppCSPS.SPSUserVar.u8_StepChain[0]++;
        }
     	stAppCSPS.SPSUserVar.u8_StepChain[0]++;
    	stAppCSPS.SPSUserVar.u16_Timer5ms[0]=100;                   // 0.5s -> 5ms x 100 / Timeout
      }
      //if (stAppCSPS.SPSUserVar.enDrvState <= eMS_HOLD)              // im Stop ausgefhrt
      break;
    }

    // Goto-Befehl zu Pos i32_PosMerker[0]
    case 1:
    {
      if (stAppCSPS.SPSCallFunctions.GotoFuncP(stAppCSPS.SPSUserVar.i32_PosMerker[0],eGOTO_um_01deg)==eMS_OK) // Call Goto Function
      {
    	stAppCSPS.SPSUserVar.u8_Dout.ucAllBits |= SET_DO2_PNP_MSK;  // Set Output 2
        stAppCSPS.SPSUserVar.u8_StepChain[0]=0;
    	stAppCSPS.SPSUserVar.u16_Timer5ms[0]=800;                   // 4s -> 5ms x 400
      }
      break;
    }
    // Homing
    case 2:
    {
      if (stAppCSPS.SPSUserVar.enDrvState <= eMS_HOLD)               // warten bis Stop ausgefhrt
      {
    	stAppCSPS.pHomingCntrl->enMode = Homing_DIx_gHi_sm | 3;      // Din3 goes Hi / Single Move
    	stAppCSPS.pHomingCntrl->u16_Timeout_ms = 5000;               // damit wir auch Zeit haben
    	if (stAppCSPS.SPSCallFunctions.HomingPrepFuncP(-50)==eMS_OK) // Call Homing Function
        {
    	  stAppCSPS.SPSUserVar.u8_StepChain[0]++;
        }
      }
      break;
    }
    // Homing-Move end, ..speichern auslsen.. hier ungefragt !!
    case 3:
    {
      if (stAppCSPS.SPSUserVar.enAppState == App_RUN_SPS)           // Homing ist beendet
      {
      	stAppCSPS.pHomingCntrl->enMode = Homing_StoreHome;
      	if (stAppCSPS.SPSCallFunctions.HomingPrepFuncP(0)==eMS_OK) // Call Homing Function
        {
      	  stAppCSPS.SPSUserVar.u8_StepChain[0]=0;
        }
      }
      break;
    }
    // Default Schritt ist quasi Delay Schritt, d.h. nach Ablauf wird Schrittzhler erhht
    default:
    {
      if (stAppCSPS.SPSUserVar.u16_Timer5ms[0]==0)            // wenn Timer bei Null, nchster Schritt
      {
        //stAppCSPS.SPSUserVar.u8_StepChain[0]++;
      }
      break;
    }
  }//..switch end
}

void LOCATEUSER AppCSPS_USER_SEQ_STANDARD_2(void)
{
  stAppCSPS.SPSUserVar.u8_Merker[0] = (stAppCSPS.SPSUserVar.u8_Din.ucAllBits & 0x1E);   // aktuellen Zustand speichern
}

/************************************************************************************************/
/*!
 *  \brief     Eigentlichen SPS-USER Funktion ( Fehlerablauf ) // Leer Vorlage
 *  \details
 *  \author    Michael Zimmermann
 *  \version   1.0
 *  \date      28.6.2019
 *  \copyright ADLOS
*   \test
*   \param
*   \return
**************************************************************************************************/
void LOCATEUSER AppCSPS_USER_SEQ_ERROR(void)
{
  mNOP();
  mNOP();
  mNOP();
  mNOP();
}

#ifdef fFeCANRXTX
void LOCATEUSER AppCSPS_USER_SEQ_CAN_RX_Event(tCANDATA* pRxData, UI_8 Datalength)
{
}
#endif
#endif  //Demo2


/************************************************************************************************/
/* Testgefss */
/************************************************************************************************/
#ifdef xDEBUG_INCLUDE_SPS_DEMOx3
void LOCATEUSER AppCSPS_USER_SEQ_STANDARD_1(void)
{
  switch (stAppCSPS.SPSUserVar.u8_StepChain[0])
{
   case 0:
   {
if ((stAppCSPS.SPSUserVar.u8_Din.ucAllBits & 0x1F) == 0x00)
{
   stAppCSPS.SPSUserVar.u8_StepChain[0]++;
}
else
{
   stAppCSPS.SPSUserVar.u8_StepChain[0] = 3;
}
      break;
   }
   case 1:
   {
if (stAppCSPS.SPSCallFunctions.RotateFuncP(1000) == eMS_OK)
{
   stAppCSPS.SPSUserVar.u8_StepChain[0]++;
}
      break;
   }
   case 2:
   {
   stAppCSPS.SPSUserVar.u8_StepChain[0] = 0;
      break;
   }
   case 3:
   {
if (stAppCSPS.SPSCallFunctions.RotateFuncP(0) == eMS_OK)
{
   stAppCSPS.SPSUserVar.u8_StepChain[0]++;
}
      break;
   }
   case 4:
   {
   stAppCSPS.SPSUserVar.u8_StepChain[0] = 0;
      break;
   }
case 5:
{
   stAppCSPS.SPSUserVar.u8_StepChain[0] = 0;
   break;
}
default:
{
   if (stAppCSPS.SPSUserVar.u16_Timer5ms[0] == 0)
   {
      stAppCSPS.SPSUserVar.u8_StepChain[0]++;
   }
}
}
}

void LOCATEUSER AppCSPS_USER_SEQ_STANDARD_2(void)
{
  mNOP();
}
void LOCATEUSER AppCSPS_USER_SEQ_ERROR(void)
{
   mNOP();
}
#endif

#ifdef xDEBUG_INCLUDE_SPS_DEMOxGEMU
/************************************************************************************************/
/*!
 *  \brief     SPS-USER Funktion / Demo <2>
 *  \details   Din-Muster <xxx0'011x> = Homing auf DIN3.....
 *             Din-Muster <xxx1'000x> = GotoPos 180
 *             Din-Muster <xxx0'100x> = GotoPos Pos+90
 *             Din-Muster <xxx0'001x> = GotoPos 0
 *             Merker[0] als Flankenmerker !
 *  \author    Michael Zimmermann
 *  \version   1.0
 *  \date      28.6.2019
 *  \copyright ADLOS
*   \test
*   \param
*   \return
**************************************************************************************************/
//#pragma GCC optimize("O0")
void LOCATEUSER AppCSPS_USER_SEQ_STANDARD_1(void)
{
  // -- Homing Starten wenn Din2= Hi, resp Pos Flanke
  if (stAppCSPS.SPSUserVar.u8_Din.ucAllBits & stAppCSPS.SPSUserVar.u8_DinChange.ucAllBits & 0x04)
  {
   stAppCSPS.SPSUserVar.u8_StepChain[0]=4;
  }

  // -- Schrittvar ausgeproggt
  switch (stAppCSPS.SPSUserVar.u8_StepChain[0])
  {
    // je nach Eingangskombination Zielposition zuweisen
    case 0:
    {
      // -- Homing Starten wenn Din2= Hi, resp Pos Flanke
      if (stAppCSPS.SPSUserVar.u8_Din.ucAllBits & stAppCSPS.SPSUserVar.u8_DinChange.ucAllBits & 0x04)
      {
      stAppCSPS.SPSUserVar.u8_StepChain[0]=4;
      }
      // -- sonst auf Position fahren
      else if (stAppCSPS.SPSUserVar.enDrvState <= eMS_HOLD)
      {
      stAppCSPS.SPSUserVar.u8_StepChain[0]=1;
      }
      break;
    }

    // Goto-Befehl zu Pos i32_PosMerker[0]
    case 1:
    {
      // Positionswert rechnen
      stAppCSPS.SPSUserVar.i32_PosMerker[0] = 2063;
      stAppCSPS.SPSUserVar.i32_PosMerker[0] *= stAppCSPS.SPSUserVar.u16_Ain;
      stAppCSPS.SPSUserVar.i32_PosMerker[0] >>= 8;
      stAppCSPS.SPSUserVar.u16_Merker[1]   = stAppCSPS.SPSUserVar.i32_PosMerker[0] & 0x0000FFFF;
      // gengend nderung
      if (stAppCSPS.SPSUserVar.u16_Merker[1] >= stAppCSPS.SPSUserVar.u16_Merker[2])
      {
      if ((stAppCSPS.SPSUserVar.u16_Merker[1] - stAppCSPS.SPSUserVar.u16_Merker[2]) < 20)
      {
         break;
      }
      }
      else if ((stAppCSPS.SPSUserVar.u16_Merker[2] - stAppCSPS.SPSUserVar.u16_Merker[1]) < 20)
      {
      break;
      }
      // Pos fahren
      if (stAppCSPS.SPSCallFunctions.GotoFuncP(stAppCSPS.SPSUserVar.u16_Merker[1],eGOTO_um_01deg)==eMS_OK) // Call Goto Function
      {
      stAppCSPS.SPSUserVar.u16_Merker[2] = stAppCSPS.SPSUserVar.u16_Merker[1];
      stAppCSPS.SPSUserVar.u8_StepChain[0]++;
      stAppCSPS.SPSUserVar.u16_Timer5ms[0]=100;                   // 0.5s -> 5ms x 200
      }
      break;
    }
    case 3:
    {
      stAppCSPS.SPSUserVar.u8_StepChain[0]=0;
      break;
    }


    // ----------  Homing -------------------------------------------
    case 4:
    {
      if (stAppCSPS.SPSCallFunctions.RotateFuncP(0)==eMS_OK)   // Stop
      {
        stAppCSPS.SPSUserVar.u8_StepChain[0]++;                 // next Step
      }
      break;
    }
    case 5:
    {
      if (stAppCSPS.SPSUserVar.enDrvState <= eMS_HOLD)               // warten bis Stop ausgefhrt
      {
      stAppCSPS.pHomingCntrl->enMode = Homing_tilSTALL | 0x05;     // Fahren auf Stall  5/16 Kraft
      stAppCSPS.pHomingCntrl->u16_Timeout_ms = 6000;               // damit wir auch Zeit haben
      if (stAppCSPS.SPSCallFunctions.HomingPrepFuncP(-80)==eMS_OK) // Call Homing Function -100rpm
        {
        stAppCSPS.SPSUserVar.u8_StepChain[0]++;
        }
      }
      break;
    }
    // Homing-Move end, ..speichern auslsen.. hier ungefragt !!
    case 6:
    {
      if (stAppCSPS.SPSUserVar.enAppState == App_RUN_SPS)           // Homing ist beendet
      {
         stAppCSPS.pHomingCntrl->enMode = Homing_StoreHome;
         if (stAppCSPS.SPSCallFunctions.HomingPrepFuncP(0)==eMS_OK) // Call Homing Function
        {
           stAppCSPS.SPSUserVar.u8_StepChain[0]=0;
        }
      }
      break;
    }
    // Default Schritt ist quasi Delay Schritt, d.h. nach Ablauf wird Schrittzhler erhht
    default:
    {
      if (stAppCSPS.SPSUserVar.u16_Timer5ms[0]==0)            // wenn Timer bei Null, nchster Schritt
      {
        stAppCSPS.SPSUserVar.u8_StepChain[0]++;
      }
      break;
    }
  }//..switch end
}

void LOCATEUSER AppCSPS_USER_SEQ_STANDARD_2(void)
{
   // Ausgangswert fr AOut rechnen
   stAppCSPS.SPSUserVar.i32_PosMerker[0] = stAppCSPS.SPSUserVar.i32_ActualPos_um_01deg;
   stAppCSPS.SPSUserVar.i32_PosMerker[0] *= 127;
   stAppCSPS.SPSUserVar.i32_PosMerker[0] >>= 10;
   stAppCSPS.SPSUserVar.u16_Merker[0]   = stAppCSPS.SPSUserVar.i32_PosMerker[0] & 0x0000FFFF;
}

/************************************************************************************************/
/*!
 *  \brief     Eigentlichen SPS-USER Funktion ( Fehlerablauf ) // Leer Vorlage
 *  \details
 *  \author    Michael Zimmermann
 *  \version   1.0
 *  \date      28.6.2019
 *  \copyright ADLOS
*   \test
*   \param
*   \return
**************************************************************************************************/
void LOCATEUSER AppCSPS_USER_SEQ_ERROR(void)
{
  mNOP();
  mNOP();
  mNOP();
  mNOP();
}

#ifdef fFeCANRXTX
void LOCATEUSER AppCSPS_USER_SEQ_CAN_RX_Event(tCANDATA* pRxData, UI_8 Datalength)
{
}
#endif
//#pragma GCC optimize("Os")
#endif  //Demo2





