/*************************************************************************************************/
/*! \file
\ingroup Appl
\brief    Motor-APP / L2 SPS-Integration / USER Seite

- Integration kundenspezifischer CMDs
- bentigt spezielles Linker Skript

CREATED
\date     28.6.2019
\version  1.0
\author   Michael Zimmermann
\remarks  -

<hr>MODIFIED
\date    17.9.2019
\version 1.1.0.0
\author  Michael Zimmermann
\remarks angepasstes Daten-Sharing / FlexUser

<hr>MODIFIED
\date    15.11.2019
\version 1.2.0.0
\author  Michael Zimmermann
\remarks BugFix im FlexUser Bereich UI_32->SI_32... K11..Benennung Aout gendert

<hr>MODIFIED
\date    9.3.2020
\version 1.3.0.0
\author  Michael Zimmermann
\remarks New, 32-Bit Struktur fr Daten von ComSchnittstelle zugefgt <st32_ComData>, 16-Byte UserNonVolatile erffnet
\remarks fr neuere Firmwaren Feature Defines gemss Datenbank setzen

<hr>MODIFIED
\date    8.4.2020
\version 1.3.1.0
\author  Michael Zimmermann
\remarks Change: Substrukturen von K11a/K11b angepasst

<hr>MODIFIED
\date    2.6.2020
\version 1.4.0.0
\author  Michael Zimmermann
\remarks Change: Strukturen fr CAN ( K17c) erweitert

<hr>MODIFIED
\date    6.8.2020
\version 1.4.0.1
\author  Michael Zimmermann
\remarks Change: Fix wegen RAM-Mapping der Funktionen K11/K17c ... Switch fFeFillDummyFunc

COPYRIGHT  2020, ADLOS AG, FL-9496 Balzers<hr>
**************************************************************************************************/
#ifndef L2_APPC_SPS_USER_H
#define L2_APPC_SPS_USER_H

#include "L0_TA_datatypes_GCC_ARM.h"                                                    // uC Include & Std-TRIDONC Type - Definitions

// ----------- Anpassungen dass Compiler Aufruf aus Manager geht ------------
#ifdef xOnlySPSpart
  #include "L0_KannCSPS_exTypes.h"
  #define  xDEBUG_INCLUDE_SPS_DEMO
#else
  #include "L0_IncludeFiles.h"
  #include "L1_PosRegulator.h"
  #include "L2_Errormanager.h"
  #include "L2_APP.h"
#endif

// ----------- Version dependent Feature Control

// --------- Modul Einstellungen --------------------------------

// --------- Linker Einstellungen --------------------------------
#define LOCATEUSCNTRL __attribute__((__section__(".mycntrlsection")))
#define LOCATEUSER    __attribute__((__section__(".myusersection")))
#define LOCATEVARSPS  __attribute__((section (".myramfunctable")))

// --------- Globale Typendefinitionen --------------------------
//! Funktionspointer... wegen rckwrtskompatibilitt auf && zurckgewechselt statt ||
#if (defined(fFeCOMCMD) && defined(fFeNVVar) && defined(fFeSpeedLim))                           //!< muss nicht zwingend jedes enthalten sein, da unten als union gelst
#define u8STD_FUNC_CNT           9                                                              //!< Anzahl effektive Funktionszeiger, aktuelles Maximum
#else
#define u8STD_FUNC_CNT           6                                                              //!< Anzahl effektive Funktionszeiger, aktuelles Minimum
#endif
typedef tenMOTERRCODE (*tROTATE_FuncP) (SI_16 iTargetSpeed01rpm_Motor);                         //!< Pointer to Rotate Funktion <L1>
typedef tenMOTERRCODE (*tGOTO_FuncP)   (SI_32 iTarget, tenGOTOPOSMODES enGotoMode);             //!< Pointer to GOTO Funktion   <L1>
typedef tenMOTERRCODE (*tHomingPrepare)(SI_8 i8Speedrpm);                                       //!< Pointer to Homing Prepare Funktion [L2]
typedef tenMOTERRCODE (*tDrvChgSetting)(UI_8 ParmIdx, UI_32 ParamValue);                        //!< Pointer to AppCSPS_UserMotorDrv_SetNV_Value
typedef void          (*tSetDigiOuts)  (UI_8 u8_OutputState);                                   //!< Pointer to Digital-Output Set
typedef bool          (*tSendUserMsg)  (UI_32 ui32_UserData);                                   //!< Pointer to SendUser-Message
typedef tenMOTERRCODE (*tStoreUserData)(void);                                                  //!< Pointer to AppCSPS_UserStoreNonVolatileData(void)
typedef void          (*tDrvSetMaxSpeed) (UI_8 u8_Speed_hPct);                                  //!< Pointer to Reg_Drv_SetTempMaxSpeed(UI_8 u8_Speed_hPct)

#ifdef fFeCANRXTX
typedef void          (*tUserCAN_RxEvent) (tCANDATA* pRxData, UI_8 Datalength);                 //!< Pointer to AppCSPS_USER_SEQ_CAN_RX_Event(tCANDATA* pRxData, UI_8 Datalength)
typedef bool          (*tCANSendData)     (tCANDATA* pTxData, UI_8 Datalength);                 //!< Pointer to bool AppCSPS_UserCAN_PDO4_Send(tCANDATA* pTxData, UI_8 Datalength)
#endif

#ifdef fFeCOMMoveEnable
typedef void          (*tUserCOMMoveAllow) (bool bMoveAllow);                                   //!< Pointer to AppCSPS_USER_COM_MoveAllow(bool bMoveAllow)
#endif

//! Status von SPS System
typedef enum
{
  eSPS_Powerup = 0,                     //!< SPS ist im Powerup
  eSPS_INIT = 1,                        //!< SPS wird initialisiert

  eSPS_OFF      = 0x10,                 //!< SPS ist aus, zB weil kein gltiger Block vorliegt
  eSPS_DOWNLOAD = 0x20,                 //!< SPS Code wird heruntergeladen

  eSPS_RUN    = 0x80,                   //!< SPS luft
  eSPS_BREAK  = 0x81,                   //!< SPS gestoppt steht in Breakpoint
}
tenSPS_State;

//! Funktionspointer Struktur, erlaubt es dem User SPS Teil Funktionen aus unserem Bereich aufzurufen
typedef union DO_ATTR_PACK DO_ALLIGN4
{
  struct
  {
    tROTATE_FuncP  RotateFuncP;        //!< R: effektiver Funktionspointer auf Rotate
    tGOTO_FuncP    GotoFuncP;          //!< R: effektiver Funktionspointer auf Goto
    tHomingPrepare HomingPrepFuncP;    //!< R: effektiver Funktionspointer auf HomingPrepare
    tDrvChgSetting DrvChgSetting;      //!< R: effektiver FunktionsPointer auf DrvSetting
    tSetDigiOuts   SetDigitalOuts;     //!< R: effektiver FunktionsPointer auf DigitalOutput Setting
    tFuncP         DrvFastStop;        //!< R: effektiver FunktionsPointer auf MotorDrv_DoFastStop()
    #ifdef fFeCOMCMD
    tSendUserMsg   SendComUserMsg;     //!< R: effektiver FunktionsPointer auf Rs232_SendUserMessage
    #elif defined(fFeFillDummyFunc)
    tFuncP         DummyFuncSendComUserMsg;
    #endif
    #ifdef fFeNVVar
    tStoreUserData StoreUserData;      //!< R: effektiver FunktionsPointer auf AppCSPS_UserStoreNonVolatileData()
    #elif defined(fFeFillDummyFunc)
    tFuncP         DummyFuncStoreUserData;
    #endif
    #ifdef fFeSpeedLim
    tDrvSetMaxSpeed DrvSetMaxSpeed;    //!< R: effektiver FunktionsPointer auf Reg_Drv_SetTempMaxSpeedhPct
    #elif defined(fFeFillDummyFunc)
    tFuncP         DummyFuncDrvSetMaxSpeed;
    #endif
    #ifdef fFeCANRXTX
    tCANSendData   CANopen_PDO4_Send;  //!< R: effektiver Funktionspointer auf AppCSPS_UserCAN_PDO4_Send(tCANDATA* pTxData, UI_8 Datalength)
    #elif defined(fFeFillDummyFunc)
    tFuncP         DummyFuncCANopen_PDO4_Send;
    #endif
    #ifdef fFeCOMMoveEnable
    tUserCOMMoveAllow  COM_MoveAllow;  //!< R: effektiver Funktionspointer auf AppCSPS_USER_COM_MoveAllow(bool bMoveAllow)
    #elif defined(fFeFillDummyFunc)
    tFuncP         DummyFuncCOM_MoveAllow;
    #endif
  };
  tFuncP FuncPtr[u8STD_FUNC_CNT];      //!< R: allgemeiner Funktionspointer
}
tCSPSFUNCTIONS;

//! SPS-Controllblock structure
typedef struct
{
  tFuncP         UserStandardFuncPtr_1;//!< R: Standard Call, Slot >1<
  tFuncP         UserStandardFuncPtr_2;//!< R: Standard Call, Slot >2<
  tFuncP         UserErrorFuncPtr;     //!< R: FehlerFunktion
  UI_32          u32_BlkChkSum;        //!< R: Checksumme ber den gesamten Block
  #ifdef fFeEXTRAFUNC
  #ifdef fFeCANRXTX
  tUserCAN_RxEvent UserRxEventFuncPtr; //!< R: Communication Rx-Event Function Pointer
  #endif
  tFuncP         UserFuncReserved[7];  //!< R: Reserved, Placeholders for Future use
  #endif
}
tAPPCSPS_CNTRL;

//! SPS-Info structure
typedef union DO_ATTR_PACK
{
  UI_32          u32Info[8];           //!< R: Infodaten Total 8x4Byte = 32-Byte
  struct
  {
     UI_16       u16_UserPrgVersion;   //!< R: User-Programmversion
     UI_16       u16_UserReserved;     //!< R: User-Reserviert, Padding von Geschichte
     UI_8        u8_UserPrgTxt[28];    //!< R: User-Programmbeschreibung
  }su;
}
tAPPCSPS_INFO;

//! SPS-Controllblock structure, inklusive User Beschreibung
typedef struct
{
  tAPPCSPS_CNTRL stCntrl;              //!< R: effektiver Control-Part
  tAPPCSPS_INFO  stInfo;               //!< R: effektiver Info-Part
}
tAPPCSPS_CNTRLBLK;

//! Serial-Data Packet got from Df-CMD, access in different possibilities
typedef union DO_ATTR_PACK DO_ALLIGN4
{
  SI_32 i32_data;                      //!< RW: full data, 32-Bit-signed
  UI_32 u32_data;                      //!< RW: full data, 32-Bit-unsigned
  struct
  {
    UI_8 u8_CMD;                       //!< RW: first Byte interpreted as COMMAND
    UI_8 u8_Data;                      //!< RW: Second Byte interpreted as data
    UI_16 u16_Data;                    //!< RW: Data interpreted as UI_16
  };
  struct
  {
    SI_8 i8_CMD;                       //!< RW: first Byte interpreted as COMMAND
    SI_8 i8_Data;                      //!< RW: Second Byte interpreted as data
    SI_16 i16_Data;                    //!< RW: Data interpreted as UI_16
  };
}
tCSPSCOMRXDATA;

#define xFLEXSIZE32    5                         //!< Anzahl 32Bit Vars Total im Flex User Bereich !
#define xFLEXSIZE32NV  4                         //!< Anzahl 32Bit Vars Total im NonVolatile Flex User Bereich !

//! K17a, vorbelegter FelxBereich durch K17a, Vorbelegung am Schluss
typedef struct DO_ATTR_PACK DO_ALLIGN4
{
  UI_16          u16_Free[(xFLEXSIZE32*2)-1];    //!< .. Pading damit die zugeteilten im Flex am Schluss zu liegen kommen
  UI_16          u16_Ain0to24_8d86mVpE;          //!< R: [8.86 mV/E]  <EVA> Abbild, Analogwert 0..20V im EVA, allenfalls gefiltert
}
tDepK17A;

//! K17c, vorbelegter FelxBereich durch K17c, Vorbelegung am Schluss
typedef struct DO_ATTR_PACK DO_ALLIGN4
{
  UI_16          u16_Free[(xFLEXSIZE32*2)-1];    //!< .. Pading damit die zugeteilten im Flex am Schluss zu liegen kommen
  UI_16          u16_Ain0to24_8d86mVpE;          //!< R: [8.86 mV/E]  <EVA> Abbild, Analogwert 0..20V im EVA, allenfalls gefiltert
}
tDepK17C;

#if (defined(fFeCOMCMD) && defined(fFeNVVar) && defined(fFeSpeedLim))
//! K11a, vorbelegter FlexBereich durch K11a, Vorbelegung am Schluss
typedef struct DO_ATTR_PACK DO_ALLIGN4
{
  UI_16          u16_Free[(xFLEXSIZE32*2)-3];    //!< .. Pading damit die zugeteilten im Flex am Schluss zu liegen kommen
  UI_16          u16_Aout4to20mA_0d01mApE;       //!< R: [0.01 mA/E]     <EVA> Abbild, Analogwert 4..20mA Ausgang
  UI_16          u16_AinMagSns_0d806mVpE;        //!< R: [0.806 mV/E]    <EVA> Abbild, Analogwert Magnet-Sensor EVA
  UI_16          u16_Ain4to20mA_0d01mApE;        //!< R: [0.01 mA/E]     <EVA> Abbild, Analogwert 4..20mA EVA, allenfalls gefiltert
}
tDepK11A;
#else
//! K11a-Alt, vorbelegter FlexBereich durch K11a, Vorbelegung am Schluss (Rckwrtskopatibilitt)
typedef struct DO_ATTR_PACK DO_ALLIGN4
{
  UI_16          u16_Free[(xFLEXSIZE32*2)-3];    //!< .. Pading damit die zugeteilten im Flex am Schluss zu liegen kommen
  UI_16          u16_Aout4to20mA_0d01mApE;       //!< R: [0.01 mA/E]     <EVA> Abbild, Analogwert 4..20mA Ausgang
  UI_16          u16_AinMagSns_0d806mVpE;        //!< R: [0.806 mV/E]    <EVA> Abbild, Analogwert Magnet-Sensor EVA
  UI_16          u16_Ain4to20mA_0d016mApE;       //!< R: [0.01611 mA/E]  <EVA> Abbild, Analogwert 4..20mA EVA, allenfalls gefiltert
}
tDepK11A;
#endif

//! K11a, vorbelegter FelxBereich durch K11a, Vorbelegung am Schluss
typedef struct DO_ATTR_PACK DO_ALLIGN4
{
  UI_16          u16_Free[(xFLEXSIZE32*2)-3];    //!< .. Pading damit die zugeteilten im Flex am Schluss zu liegen kommen
  UI_16          u16_Aout0to10V_1mVpE;           //!< R: [1mV/E]         <EVA> Abbild, Analogwert 0..10V Ausgang
  UI_16          u16_AinMagSns_0d806mVpE;        //!< R: [0.806 mV/E]    <EVA> Abbild, Analogwert Magnet-Sensor EVA
  UI_16          u16_Ain0to10v_1mVpE;            //!< R: [1mV/E]         <EVA> Abbild, Analogwert 0..10V im EVA, allenfalls gefiltert
}
tDepK11B;

//! SPS-vordefinierte USER-Variabeln
typedef struct DO_ATTR_PACK DO_ALLIGN4
{
  unDIN_Infot    u8_Din;                //!< R: <EVA> Abbild der digitalen Eingnge
  unDIN_Infot    u8_DinChange;          //!< R: <EVA> nderungen seit dem letzten Mal
  unDOUTS12t     u8_Dout;               //!< R: <EVA> Abbild der digitalen Ausgnge
  tenAPPSTATES   enAppState;            //!< R: <EVA> aktueller Zustand APP
  SI_32          i32_ActualPos_um_01deg;//!< R: <EVA> aktuelle Position
  tERRBITS       u16_Errorbits;         //!< R: <EVA> Abbbild Fehlerbits
  tenMOTSTATES   enDrvState;            //!< R: <EVA> aktueller Zustand DRV
  SI_8           i8_VARX;               //!< R: 1x allgemeine 8-Bit VAR [8-Bit]
  UI_16          u16_Timer5ms[4];       //!< R: 4x 5ms Timer [16-Bit]
  UI_8           u8_StepChain[4];       //!< R: 4x Schrittketten- Vars [8-Bit]
  SI_32          i32_PosVAR_um_01deg[3];//!< RW: 3x zustzliche Psoitionsmerker ..zB um Endanschlge zu merken
  union                                 //       5x 4Bytes = 20 Byte, Flex User Section, Overlapped !
  {
    tDepK17A     K17a;                       //!< R: <EVA> vom 17a
    tDepK17C     K17c;                       //!< R: <EVA> vom 17c
    tDepK11A     K11a;                       //!< R: <EVA> vom 11a
    tDepK11B     K11b;                       //!< R: <EVA> vom 11b
    UI_32        u32_UserVar[xFLEXSIZE32];   //!< R: je nach Steuerung freies USER Feld 32-Bit UI Zugriff
    SI_32        i32_UserVar[xFLEXSIZE32];   //!< R: je nach Steuerung freies USER Feld 32-Bit SI Zugriff
    UI_16        u16_UserVar[xFLEXSIZE32*2]; //!< R: je nach Steuerung freies USER Feld 16-Bit UI Zugriff
    SI_16        i16_UserVar[xFLEXSIZE32*2]; //!< R: je nach Steuerung freies USER Feld 16-Bit SI Zugriff
    UI_8         u08_UserVar[xFLEXSIZE32*4]; //!< R: je nach Steuerung freies USER Feld 8-Bit UI Zugriff
    SI_8         i08_UserVar[xFLEXSIZE32*4]; //!< R: je nach Steuerung freies USER Feld 8-Bit SI Zugriff
  }
  KxxFlexUser;
  #ifdef fFeCOMCMD
  tCSPSCOMRXDATA st32_ComData;               //!< RW: from Serial Port received data Command <Df>, shall be cleared by USER !
  #endif
  #ifdef fFeNVVar
  union                                 //       5x 4Bytes = 20 Byte, Flex User Section, Overlapped !
  {
    UI_32        u32_UserVar[xFLEXSIZE32NV];     //!< R: USER Feld 32-Bit UI Zugriff, nonVolatile
    SI_32        i32_UserVar[xFLEXSIZE32NV];     //!< R: USER Feld 32-Bit SI Zugriff, nonVolatile
    UI_16        u16_UserVar[xFLEXSIZE32NV*2];   //!< R: USER Feld 16-Bit UI Zugriff, nonVolatile
    SI_16        i16_UserVar[xFLEXSIZE32NV*2];   //!< R: USER Feld 16-Bit SI Zugriff, nonVolatile
    UI_8         u08_UserVar[xFLEXSIZE32NV*4];  //!< R: USER Feld 8-Bit UI Zugriff, nonVolatile
    SI_8         i08_UserVar[xFLEXSIZE32NV*4];   //!< R: USER Feld 8-Bit SI Zugriff, nonVolatile
  }
  UserNonVolatile;
  #endif
}
tAPPCSPS_USERVAR;

//! USER - SPS Daten structure
typedef struct
{
  tCSPSFUNCTIONS     SPSCallFunctions;     //!< R:  Tabelle mit den zur Verfgung stehenden Funktionen
  tHOMINGCNTRL*      pHomingCntrl;         //!< R:  Zeiger auf Homing Struktur
  tenSPS_State       State;                //!< RW: SPS Status
  tAPPCSPS_USERVAR   SPSUserVar;           //!< R:  SPS Uservariabeln
}
tAPPCSPS;

// -------------- Globale Variabeln -------------------
extern tAPPCSPS LOCATEVARSPS stAppCSPS;

// --------- Exported functions -----------------
void LOCATEUSER AppCSPS_USER_SEQ_STANDARD_1(void);
void LOCATEUSER AppCSPS_USER_SEQ_STANDARD_2(void);
void LOCATEUSER AppCSPS_USER_SEQ_ERROR(void);

#ifdef fFeCANRXTX
void LOCATEUSER AppCSPS_USER_SEQ_CAN_RX_Event(tCANDATA* pRxData,UI_8 Datalength);
#endif

#endif
