/*************************************************************************************************/
/*! \file
\ingroup Appl
\brief    Motor-APP / L2 SPS-Integration / USER Seite

- Integration kundenspezifischer CMDs
- bentigt spezielles Linker Skript

CREATED
\date     28.6.2019
\version  1.0
\author   Michael Zimmermann
\remarks  -

<hr>MODIFIED
\date    -
\version -
\author  -
\remarks -

COPYRIGHT  2019, ADLOS AG, FL-9496 Balzers<hr>
**************************************************************************************************/

#include "L0_TA_datatypes_GCC_ARM.h"                                      // uC Include & Std-TRIDONC Type - Definitions
#include "stm32f0xx.h"
#include "L0_KannMotDrvTypes.h"
#include "L2_APPC_SPS_User.h"

// ---- Internal Definitions ----------------------------

// ---- SPS Control-Block ----------------------
volatile const tAPPCSPS_CNTRLBLK LOCATEUSCNTRL  CNTRLBLKINIT = { {AppCSPS_USER_SEQ_STANDARD_1,                         // Func Pointer
		                                                            AppCSPS_USER_SEQ_STANDARD_2,                         // Func Pointer
		                                                            AppCSPS_USER_SEQ_ERROR,                              // Func Pointer
		                                                            0xFFFFFFFF},                                         // Checksumme
		                                                            0,                                                   // Version
                                                                   {"--DEMO Ventil--"} };                                    // USER Text/ Beschreibung

// ---- Globale Variabeln lsen -----------------
tAPPCSPS LOCATEVARSPS stAppCSPS;

// **HEADER_END**             nicht lschen
								   
/************************************************************************************************/
/*!
 *  \brief     SPS-USER Funktion / Demo <Ventil>
 *  \details   Ventil Muster, K11a mit Linearem Antrieb ( Pos Werte sind in um )

 *             Homing auf DIN1, positive Flanke, Oder MagnetAbzug von Sensor
 *             4-6mA (Close=Ziel=0) Nachdrckemode, alle 5s
 *       6-20mA (Open=Ziel= gerechnet abhngig von Ain)
 *
 *             ..KxxFlexUser.i32_TUserVar[0] verwendet als Berechnungs-HilfsVariabel
 *             ..KxxFlexUser.u08_TUserVar[4] verwendet als Zustandsmerker fr Externen Homing-Magneten (8-Bit-Merker)
 *             ..KxxFlexUser.u08_TUserVar[5] Hilfsmerker fr Mode (0:off; 1:Close; 2:Open)
 *             ..KxxFlexUser.u16_UserVar[4]  t-1 Merker von 4..20mA Input
 *             ..KxxFlexUser.i16_UserVar[3]  Absolute Vernderung vom 4..20mA zum t-1 Wert
 *             ..i32_PosVAR_um_01deg[0] gerechnete Zielposition in um
 *             ..i32_PosVAR_um_01deg[1] 2.Endposition ( optional, aktuell ausgeklammert)
 *

 *  \author    Michael Zimmermann
 *  \version   1.0
 *  \date      23.9.2019
 *  \copyright ADLOS
*   \test
*   \param
*   \return
**************************************************************************************************/
typedef enum
{
  eUSTATE0_INIT = 0,                                                //!<  Startup-Init State
  eUSTATE1_CHECK = 1,                                               //!<  Check-Triage State

  eUSTATE2_CALC_OPOS = 2,                                           //!<  Calculate OPenPos according 4..20mA
  eUSTATE3_START_GOTO = 3,                                          //!<  GotoPos Initiate/Start
  eUSTATE4_DELAY = 4,                                               //!<  cooperative Dealy according Setting Timer[0]
  eUSTATE5_WAIT_MOVEEND = 5,                                        //!<  wait until drive stops

  eUSTATE6_PRESS_CLOSE = 6,                                         //!<  Close to 0, until Stall

  eUSTATE10_HOME_STOPDRV = 10,                                      //!<  HOMING, first Stop drive
  eUSTATE11_HOME_INIT = 11,                                         //!<  HOMING, Init Home Movement
  eUSTATE12_HOME_ENDSTORE = 12,                                     //!<  HOMING, End and Store
}
tenUSERSTATES;



void LOCATEUSER AppCSPS_USER_SEQ_STANDARD_1(void)
{
  // ------------------------------------------------------------------------------------------------
  // --> Bestimmung Magnetsensor-Wert, je nach Sensor haben wir 11..90mV/T
  // ------------------------------------------------------------------------------------------------
  // Abstand von 1V/0mT bestimmen, dann mit 200mV Schwelle vergleichen, wenn > dann Magnet Erkannt Bit einschieben !
  // Schlussendlich wollen wir erkennnen wenn der Magnet abgezogen wird, d.h. wir Triggern auf Muster 0b1111'1000 d.h. 5xHi /3xLo
  stAppCSPS.SPSUserVar.KxxFlexUser.i32_UserVar[0] = 1241 - stAppCSPS.SPSUserVar.KxxFlexUser.K11a.u16_AinMagSns_0d806mVpE;   // 1241 = 1V = 0mT
  if (stAppCSPS.SPSUserVar.KxxFlexUser.i32_UserVar[0] < 0)
  {
   stAppCSPS.SPSUserVar.KxxFlexUser.i32_UserVar[0] *= (-1);
  }
  stAppCSPS.SPSUserVar.KxxFlexUser.u08_UserVar[4] <<= 1;                                // nach links-schieben
  if (stAppCSPS.SPSUserVar.KxxFlexUser.i32_UserVar[0] > 248)                            // > 200mV
  {
   stAppCSPS.SPSUserVar.KxxFlexUser.u08_UserVar[4] |= 0x01;                        // haben Magneten, zustzlich eine 1 einschieben
  }
  
  // ------------------------------------------------------------------------------------------------
  // --> Event Reactions
  // ------------------------------------------------------------------------------------------------
  //     ON_DI1, posSlope                                  -> DoHoming         [Step=10]
  //     OR on Magnet remove sampled b1111'1000            
  if (((stAppCSPS.SPSUserVar.u8_Din.b.bDI1==1) & (stAppCSPS.SPSUserVar.u8_DinChange.b.bDI1)) ||   // DI-1, pos Slope
       (stAppCSPS.SPSUserVar.KxxFlexUser.u08_UserVar[4]==0xF8))                                   // Magnet release                              
  {
   stAppCSPS.SPSUserVar.u8_StepChain[0]=eUSTATE10_HOME_STOPDRV;
  }
  /*
  // -- DI0, negSlope                                      -> goto other End   [Step=20]
  else if ((stAppCSPS.SPSUserVar.u8_Din.b.bDI0==0) & (stAppCSPS.SPSUserVar.u8_DinChange.b.bDI0))
  {
   stAppCSPS.SPSUserVar.u8_StepChain[0]=20;
  }*/

  // ------------------------------------------------------------------------------------------------
  // --> CMD Sequence Handler
  // ------------------------------------------------------------------------------------------------
  switch (stAppCSPS.SPSUserVar.u8_StepChain[0])
  {
    // ------------------------------------------------------------------------------------------------
    // --> Startup-Step, Demo Showing how to change on fly the Torques, see Doc 100570
    // ------------------------------------------------------------------------------------------------
    case eUSTATE0_INIT:
    {
      stAppCSPS.SPSCallFunctions.DrvChgSetting(14, 0); // Param 14, Holding Torque = 0
      stAppCSPS.SPSCallFunctions.DrvChgSetting(15, 160); // Param 15, ACC Torque = 80%
      stAppCSPS.SPSCallFunctions.DrvChgSetting(16, 160); // Param 16, RUN Torque = 80%
      stAppCSPS.SPSCallFunctions.DrvChgSetting(17, 160); // Param 17, DEC Torque = 80%
	    
      stAppCSPS.SPSCallFunctions.DrvChgSetting(21, ADIN_FIL_MEDIAN | 0x03);     // Sample Filter activation ( when not Default NonVolatile Setting is used)  Debounce on DI0&DI1
      //stAppCSPS.SPSCallFunctions.DrvChgSetting(21, 0);                                // No Extra Filter selected Filtereing is DeepPass 1.O
      
      stAppCSPS.SPSUserVar.KxxFlexUser.u16_UserVar[4]=0;
      stAppCSPS.SPSUserVar.KxxFlexUser.u08_UserVar[5]=0;
      stAppCSPS.SPSUserVar.u8_StepChain[0]++;
      break;
    }
    // ------------------------------------------------------------------------------------------------
    // --> Check Ain Triage, Triage is depending on Histrory ( Hysteretic Behave )
    //     KxxFlexUser.u08_TUserVar[5] Hilfsmerker fr Mode (0:off; 1:Close; 2:Open)
    //     DI0 zeigt mit Hysterese ob der 4..20mA Eingang strom fhrt (True=ja)
    // ------------------------------------------------------------------------------------------------
    case eUSTATE1_CHECK:
    {
      if (stAppCSPS.SPSUserVar.u8_Din.b.bDI0==1)
      {
        if (stAppCSPS.SPSUserVar.KxxFlexUser.u08_UserVar[5]==0)
        {
          if (stAppCSPS.SPSUserVar.KxxFlexUser.K11a.u16_Ain4to20mA_0d016mApE >= (6 * 62))
          //if (stAppCSPS.SPSUserVar.KxxFlexUser.K11a.u16_Ain4to20mA_0d01mApE >= 600)  ab V2.5
	  {
            stAppCSPS.SPSUserVar.u8_StepChain[0]++;
            stAppCSPS.SPSUserVar.KxxFlexUser.u08_UserVar[5]=2;
          }
          else if (stAppCSPS.SPSUserVar.KxxFlexUser.K11a.u16_Ain4to20mA_0d016mApE >= (4 * 62))
          //else if (stAppCSPS.SPSUserVar.KxxFlexUser.K11a.u16_Ain4to20mA_0d01mApE >= 400)  ab V2.5
          {
            stAppCSPS.SPSUserVar.u8_StepChain[0]=eUSTATE6_PRESS_CLOSE;
            stAppCSPS.SPSUserVar.KxxFlexUser.u08_UserVar[5]=1;
          }
        }
        else if ((stAppCSPS.SPSUserVar.KxxFlexUser.u08_UserVar[5]==2) && (stAppCSPS.SPSUserVar.KxxFlexUser.K11a.u16_Ain4to20mA_0d016mApE >= ((55 * 62) / 10)))  // 5.5mA
        {
          stAppCSPS.SPSUserVar.u8_StepChain[0]++;
        }
        else if ((stAppCSPS.SPSUserVar.KxxFlexUser.u08_UserVar[5]==1) && (stAppCSPS.SPSUserVar.KxxFlexUser.K11a.u16_Ain4to20mA_0d016mApE >= ((35 * 62) / 10)))  // 3.5mA
        {
           if (stAppCSPS.SPSUserVar.KxxFlexUser.K11a.u16_Ain4to20mA_0d016mApE >= (6 * 62))
           {
             stAppCSPS.SPSUserVar.u8_StepChain[0]++;
             stAppCSPS.SPSUserVar.KxxFlexUser.u08_UserVar[5]=2;
           }
           else stAppCSPS.SPSUserVar.u8_StepChain[0]=eUSTATE6_PRESS_CLOSE;
        }
        else
        {
          stAppCSPS.SPSUserVar.KxxFlexUser.u08_UserVar[5]=0;
        }
      }
      else stAppCSPS.SPSUserVar.KxxFlexUser.u08_UserVar[5]=0;
      break;
    }

    // ------------------------------------------------------------------------------------------------
    // --> OPen / Goto Position, According AnalogInput
    // ------------------------------------------------------------------------------------------------
    //     Calc from [mA] -> Position in [um]
    case eUSTATE2_CALC_OPOS:
    {
       // --> Ainput Diff Bestimmung zum letzten Mal, Differenz Soll Absolut sein
       stAppCSPS.SPSUserVar.KxxFlexUser.i16_UserVar[3] = stAppCSPS.SPSUserVar.KxxFlexUser.K11a.u16_Ain4to20mA_0d016mApE - stAppCSPS.SPSUserVar.KxxFlexUser.u16_UserVar[4]; 
       if (stAppCSPS.SPSUserVar.KxxFlexUser.i16_UserVar[3] < 0)
       {
         stAppCSPS.SPSUserVar.KxxFlexUser.i16_UserVar[3] *= -1;
       }     
       // neue Zielpos rechnen wenn Change > 2-LSB
       if (stAppCSPS.SPSUserVar.KxxFlexUser.i16_UserVar[3]>2)
       {
         stAppCSPS.SPSUserVar.KxxFlexUser.u16_UserVar[4] = stAppCSPS.SPSUserVar.KxxFlexUser.K11a.u16_Ain4to20mA_0d016mApE;  // merken
         // .. Strom in Position umrechnen: 6mA -> 500um // 20mA zu 10'000um // dazu rechnen wir:  (Curr-6mA) * 1960/256  + 500 siehe Excel
         stAppCSPS.SPSUserVar.i32_PosVAR_um_01deg[0] = stAppCSPS.SPSUserVar.KxxFlexUser.K11a.u16_Ain4to20mA_0d016mApE;
         if (stAppCSPS.SPSUserVar.i32_PosVAR_um_01deg[0] >= (6 * 62))
         {
           stAppCSPS.SPSUserVar.i32_PosVAR_um_01deg[0] -= (6 * 62);
         }
         else stAppCSPS.SPSUserVar.i32_PosVAR_um_01deg[0]=0;
         stAppCSPS.SPSUserVar.i32_PosVAR_um_01deg[0] *= 1960;
         stAppCSPS.SPSUserVar.i32_PosVAR_um_01deg[0] >>= 8;
         stAppCSPS.SPSUserVar.i32_PosVAR_um_01deg[0] += 500;
         stAppCSPS.SPSUserVar.u8_StepChain[0]++;
       }
       break;
    }

    case eUSTATE3_START_GOTO:
    {
      // .. Pos anfahren
      if (stAppCSPS.SPSCallFunctions.GotoFuncP(stAppCSPS.SPSUserVar.i32_PosVAR_um_01deg[0],eGOTO_um_01deg)==eMS_OK) // Call Goto Function
      {
      stAppCSPS.SPSUserVar.u8_StepChain[0]++;
      stAppCSPS.SPSUserVar.u16_Timer5ms[0]=100;                   // 500ms Min Starting/Driving Time -> 5ms x 20
      }
      break;
    }
    //    GotoPos... Wait until Drive Stops, than REDO
    case eUSTATE5_WAIT_MOVEEND:
    {
      if (stAppCSPS.SPSUserVar.enDrvState <= eMS_HOLD)
      {
        stAppCSPS.SPSUserVar.u8_StepChain[0]=eUSTATE1_CHECK;
      }
      break;
    }
    
    // ------------------------------------------------------------------------------------------------
    // --> Close / Goto Position, According AnalogInput
    // ------------------------------------------------------------------------------------------------
    //    Check where Target is, depending on that, Drive to 0 or 'Homing' set some Pressure 
    case eUSTATE6_PRESS_CLOSE:
    {    
      if ((stAppCSPS.SPSUserVar.i32_PosVAR_um_01deg[0]!=0) || (stAppCSPS.SPSUserVar.i32_ActualPos_um_01deg>0))
      {
        stAppCSPS.SPSUserVar.i32_PosVAR_um_01deg[0] = 0;              // Neues Ziel = 0
        stAppCSPS.SPSUserVar.u8_StepChain[0]=eUSTATE3_START_GOTO;
      }
      else if (stAppCSPS.SPSUserVar.u16_Timer5ms[1]==0)
      {
        stAppCSPS.pHomingCntrl->enMode = Homing_tilSTALL | 0x0A;      // Fahren auf Stall  10/16 Kraft
        stAppCSPS.pHomingCntrl->u16_Timeout_ms = 1000;                // 1000ms max Time
        if (stAppCSPS.SPSCallFunctions.HomingPrepFuncP(-100)==eMS_OK) // Call Homing Function -100rpm
        {
          stAppCSPS.SPSUserVar.u16_Timer5ms[1]=1000;                  // Retry Press after 5*1000 = 5000ms
          stAppCSPS.SPSUserVar.u8_StepChain[0]=eUSTATE1_CHECK;
        }
      }
      break;
    }

    // ------------------------------------------------------------------------------------------------
    // --> DO  Homing
    // ------------------------------------------------------------------------------------------------
    //    Homing: Stop Drive
    case eUSTATE10_HOME_STOPDRV:
    {
      stAppCSPS.SPSCallFunctions.DrvFastStop();                      // direct not ramping ....
      stAppCSPS.SPSUserVar.u8_StepChain[0]++;                        // next Step  (+5ms) to Stop Drive
      break;
    }
    //    Homing: wait until stopped than Init Homing: <Mode, Speed, Torque, Timeout>
    case eUSTATE11_HOME_INIT:
    {
      if (stAppCSPS.SPSUserVar.enDrvState <= eMS_HOLD)               // warten bis Stop ausgefhrt
      {
        stAppCSPS.pHomingCntrl->enMode = Homing_tilSTALL | 0x05;     // Fahren auf Stall  5/16 Kraft
        stAppCSPS.pHomingCntrl->u16_Timeout_ms = 10000;              // 10s max Time
        if (stAppCSPS.SPSCallFunctions.HomingPrepFuncP(-100)==eMS_OK) // Call Homing Function -100rpm
        {
          stAppCSPS.SPSUserVar.u8_StepChain[0]++;
        }
      }
      break;
    }
    //    Homing END, coming back out from Homing State... Store AS Home (0) Psoition:
    case eUSTATE12_HOME_ENDSTORE:
    {
      stAppCSPS.pHomingCntrl->enMode = Homing_StoreHome;
      if (stAppCSPS.SPSCallFunctions.HomingPrepFuncP(0)==eMS_OK)     // Call Homing Function
      {
         stAppCSPS.SPSUserVar.KxxFlexUser.u16_UserVar[4]=0;          // Merker nullen da wir nun da stehen
	 stAppCSPS.SPSUserVar.u8_StepChain[0]=eUSTATE1_CHECK;
      }
      break;
    }

    // ------------------------------------------------------------------------------------------------
    // --> Missuse Homing to find other (Open Pos) ... Passive Position
    // ------------------------------------------------------------------------------------------------
    //    Homing: Stop Drive
    case 20:
    {
      stAppCSPS.SPSCallFunctions.DrvFastStop();                      // direct not ramping ....
      stAppCSPS.SPSUserVar.u8_StepChain[0]++;                        // next Step  (+5ms) to Stop Drive
      break;
    }
    //    Homing: wait until stopped than Init Homing: <Mode, Speed, Torque, Timeout>
    case 21:
    {
      if (stAppCSPS.SPSUserVar.enDrvState <= eMS_HOLD)               // warten bis Stop ausgefhrt
      {
        stAppCSPS.pHomingCntrl->enMode = Homing_tilSTALL | 0x05;     // Fahren auf Stall  5/16 Kraft
        stAppCSPS.pHomingCntrl->u16_Timeout_ms = 10000;              // 10s Timeout
        if (stAppCSPS.SPSCallFunctions.HomingPrepFuncP(+100)==eMS_OK) // Call Homing Function +100rpm
        {
          stAppCSPS.SPSUserVar.u8_StepChain[0]++;
        }
      }
      break;
    }
    //    Homing END, we could save here
    case 22:
    {
      stAppCSPS.SPSUserVar.i32_PosVAR_um_01deg[1]=stAppCSPS.SPSUserVar.i32_ActualPos_um_01deg;
      stAppCSPS.SPSUserVar.u8_StepChain[0]=1;
      break;
    }

    // ------------------------------------------------------------------------------------------------
    // --> cooperative Delay State
    // ------------------------------------------------------------------------------------------------
    //   Stays inside State until Timer[0] is at Zero
    default:
    {
      if (stAppCSPS.SPSUserVar.u16_Timer5ms[0]==0)            // wenn Timer bei Null, nchster Schritt
      {
        stAppCSPS.SPSUserVar.u8_StepChain[0]++;
      }
      break;
    }
  }//..switch end
}

void LOCATEUSER AppCSPS_USER_SEQ_STANDARD_2(void)
{
   // ------------------------------------------------------------------------------------------------
   // --> Feed Analog Output, Calculate from Pos[um] -> [mA]      Pos*127/512
   // ------------------------------------------------------------------------------------------------
   if (stAppCSPS.SPSUserVar.i32_ActualPos_um_01deg>0)
   {
     stAppCSPS.SPSUserVar.KxxFlexUser.i32_UserVar[0] = stAppCSPS.SPSUserVar.i32_ActualPos_um_01deg;
   }
   else stAppCSPS.SPSUserVar.KxxFlexUser.i32_UserVar[0]=0;
   stAppCSPS.SPSUserVar.KxxFlexUser.i32_UserVar[0] *= 127;
   stAppCSPS.SPSUserVar.KxxFlexUser.i32_UserVar[0] >>= 9;
   stAppCSPS.SPSUserVar.KxxFlexUser.K11a.u16_Aout4to20mA_0d01mApE = stAppCSPS.SPSUserVar.KxxFlexUser.i32_UserVar[0] & 0x0000FFFF;
}

/************************************************************************************************/
/*!
 *  \brief     Eigentlichen SPS-USER Funktion ( Fehlerablauf ) // Leer Vorlage
 *  \details
 *  \author    Michael Zimmermann
 *  \version   1.0
 *  \date      28.6.2019
 *  \copyright ADLOS
*   \test
*   \param
*   \return
**************************************************************************************************/
void LOCATEUSER AppCSPS_USER_SEQ_ERROR(void)
{
  mNOP();
  mNOP();
  mNOP();
  mNOP();
}