/*************************************************************************************************/
/*! \file
\ingroup Appl
\brief    Motor-APP / L2 SPS-Integration / USER Seite

- Integration kundenspezifischer CMDs
- bentigt spezielles Linker Skript

CREATED
\date     28.6.2019
\version  1.0
\author   Michael Zimmermann
\remarks  -

<hr>MODIFIED
\date    -
\version -
\author  -
\remarks -

COPYRIGHT  2019, ADLOS AG, FL-9496 Balzers<hr>
**************************************************************************************************/

#include "L0_TA_datatypes_GCC_ARM.h"                                      // uC Include & Std-TRIDONC Type - Definitions
#include "stm32f0xx.h"
#include "L0_KannMotDrvTypes.h"
#include "L2_APPC_SPS_User.h"

// ---- Internal Definitions ----------------------------

// ---- SPS Control-Block ----------------------
volatile const tAPPCSPS_CNTRLBLK LOCATEUSCNTRL  CNTRLBLKINIT = { {AppCSPS_USER_SEQ_STANDARD_1,                         // Func Pointer
		                                                            AppCSPS_USER_SEQ_STANDARD_2,                         // Func Pointer
		                                                            AppCSPS_USER_SEQ_ERROR,                              // Func Pointer
		                                                            0xFFFFFFFF},                                         // Checksumme
		                                                            0,                                                   // Version
                                                                   {"-DEMO1-"} };                                    // USER Text/ Beschreibung

// ---- Globale Variabeln lsen -----------------
tAPPCSPS LOCATEVARSPS stAppCSPS;

// **HEADER_END**             nicht lschen
								   
/************************************************************************************************/
/*!
 *  \brief     SPS-USER Funktion / Demo <1>
 *  \details   Zeitfahrt mit 100rpm <2s>, danach GotoPos 100... endlos wiederholend
 *             die Ausgnge Out1 & Out2 zeigen jewiels die Drehrichtung
 *	      Demo fr K17a Rotative geeignet 					   
 *  \author    Michael Zimmermann
 *  \version   1.0
 *  \date      28.6.2019
 *  \copyright ADLOS
*   \test
*   \param
*   \return
**************************************************************************************************/
void LOCATEUSER AppCSPS_USER_SEQ_STANDARD_1(void)
{
  // -- Schrittvar ausgeproggt
  switch (stAppCSPS.SPSUserVar.u8_StepChain[0])
  {
    // Start, Rotate Befehl +100rpm, danach Delay Laden mit 2s, nchster Schritt
    case 0:
    {
      if (stAppCSPS.SPSCallFunctions.RotateFuncP(+1000)==eMS_OK)  // Call Rotate Function
      {
    	stAppCSPS.SPSUserVar.u8_Dout.ucAllBits = SET_DO1_PNP_MSK;   // Set Output 1
    	stAppCSPS.SPSUserVar.u16_Timer5ms[0]=400;                   // 2s -> 5ms x 400
        stAppCSPS.SPSUserVar.u8_StepChain[0]++;                   // next Step = Delay
      }
      break;
    }
    // Rotate Befehl 0 = Stop
    case 2:
    {
      if (stAppCSPS.SPSCallFunctions.RotateFuncP(0)==eMS_OK)
      {
        stAppCSPS.SPSUserVar.u8_StepChain[0]++;                 // next Step
      }
      break;
    }
    // Goto-Befehl zu Pos 100, danach Delay Laden mit 4s, nchster Schritt
    case 3:
    {
      if (stAppCSPS.SPSUserVar.enDrvState <= eMS_HOLD)                        // warten bis Stop ausgefhrt
      {
        if (stAppCSPS.SPSCallFunctions.GotoFuncP(100,eGOTO_um_01deg)==eMS_OK) // Call Goto Function
        {
    	  stAppCSPS.SPSUserVar.u8_Dout.ucAllBits = SET_DO2_PNP_MSK;  // Set Output 2
    	  stAppCSPS.SPSUserVar.u16_Timer5ms[0]=800;                  // 4s -> 5ms x 400
          stAppCSPS.SPSUserVar.u8_StepChain[0]++;                  // next Step = Delay
        }
      }
      break;
    }
    // neu von vorne
    case 5:
    {
      stAppCSPS.SPSUserVar.u8_StepChain[0]=0;
      break;
    }
    // Default Schritt ist quasi Delay Schritt, d.h. nach Ablauf wird Schrittzhler erhht
    default:
    {
      if (stAppCSPS.SPSUserVar.u16_Timer5ms[0]==0)            // wenn Timer bei Null, nchster Schritt
      {
        stAppCSPS.SPSUserVar.u8_StepChain[0]++;
      }
      break;
    }
  }//..switch end
}

void LOCATEUSER AppCSPS_USER_SEQ_STANDARD_2(void)
{
  mNOP();
  mNOP();
}

/************************************************************************************************/
/*!
 *  \brief     Eigentlichen SPS-USER Funktion ( Fehlerablauf ) // Leer Vorlage
 *  \details
 *  \author    Michael Zimmermann
 *  \version   1.0
 *  \date      28.6.2019
 *  \copyright ADLOS
*   \test
*   \param
*   \return
**************************************************************************************************/
void LOCATEUSER AppCSPS_USER_SEQ_ERROR(void)
{
  mNOP();
  mNOP();
  mNOP();
  mNOP();
}





