/*************************************************************************************************/
/*! \file
\ingroup Appl
\brief    Motor-APP / L2 SPS-Integration / USER Seite

- Integration kundenspezifischer CMDs
- bentigt spezielles Linker Skript

CREATED
\date     28.6.2019
\version  1.0
\author   Michael Zimmermann
\remarks  -

<hr>MODIFIED
\date    -
\version -
\author  -
\remarks -

COPYRIGHT  2019, ADLOS AG, FL-9496 Balzers<hr>
**************************************************************************************************/

#include "L0_TA_datatypes_GCC_ARM.h"                                      // uC Include & Std-TRIDONC Type - Definitions
#include "stm32f0xx.h"
#include "L0_KannMotDrvTypes.h"
#include "L2_APPC_SPS_User.h"

// ---- Internal Definitions ----------------------------

// ---- SPS Control-Block ----------------------
volatile const tAPPCSPS_CNTRLBLK LOCATEUSCNTRL  CNTRLBLKINIT = { {AppCSPS_USER_SEQ_STANDARD_1,                         // Func Pointer
		                                                            AppCSPS_USER_SEQ_STANDARD_2,                         // Func Pointer
		                                                            AppCSPS_USER_SEQ_ERROR,                              // Func Pointer
		                                                            0xFFFFFFFF},                                         // Checksumme
		                                                            0,                                                   // Version
                                                                   {"--DEMO2--"} };                                    // USER Text/ Beschreibung

// ---- Globale Variabeln lsen -----------------
tAPPCSPS LOCATEVARSPS stAppCSPS;

// **HEADER_END**             nicht lschen
								   
/************************************************************************************************/
/*!
 *  \brief     SPS-USER Funktion / Demo <2>
 *  \details   Mixed Demo mit Positionsfahrt/ + Fahrt / und Rotate, K17a Rotative geeignet
 *	      Din-Muster <xxx0'011x> = Homing auf DIN3.....
 *             Din-Muster <xxx1'000x> = GotoPos 180
 *             Din-Muster <xxx0'100x> = GotoPos Pos+90
 *             Din-Muster <xxx0'001x> = GotoPos 0
 *
 *             stAppCSPS.SPSUserVar.i32_PosVAR_um_01deg[0]  als Zielspeicher verwendet
 *
 *  \author    Michael Zimmermann
 *  \version   1.0
 *  \date      17.9.2019
 *  \copyright ADLOS
 *  \test
 *  \param
 *  \return
**************************************************************************************************/
void LOCATEUSER AppCSPS_USER_SEQ_STANDARD_1(void)
{
  // -- Schrittvar ausgeproggt
  switch (stAppCSPS.SPSUserVar.u8_StepChain[0])
  {
    // je nach Eingangskombination Zielposition zuweisen
    case 0:
    {
      // -- Zielposition gemss Schaltermuster zuweisen
      if ((stAppCSPS.SPSUserVar.u8_Din.ucAllBits & 0x1E) ==  0x10)
      {
    	stAppCSPS.SPSUserVar.i32_PosVAR_um_01deg[0]=1800;
      }
      else if ((stAppCSPS.SPSUserVar.u8_Din.ucAllBits & 0x1E) ==  0x02)
      {
    	  stAppCSPS.SPSUserVar.i32_PosVAR_um_01deg[0]=0;
      }
      // -- bei Positiver Flanke, Gotopos CMD ausfhren lassen / Schrittmerker (+1)
      if (stAppCSPS.SPSUserVar.u8_DinChange.ucAllBits & stAppCSPS.SPSUserVar.u8_Din.ucAllBits & 0x1E)
      {
        if ((stAppCSPS.SPSUserVar.u8_Din.ucAllBits & 0x1E) ==  0x08)      // +90
        {
          stAppCSPS.SPSUserVar.i32_PosVAR_um_01deg[0]+=900;
        }
        else if ((stAppCSPS.SPSUserVar.u8_Din.ucAllBits & 0x1E) ==  0x06) // Homing
        {
          stAppCSPS.SPSUserVar.u8_StepChain[0]++;
        }
     	stAppCSPS.SPSUserVar.u8_StepChain[0]++;
    	stAppCSPS.SPSUserVar.u16_Timer5ms[0]=100;                   // 0.5s -> 5ms x 100 / Timeout
      }
      //if (stAppCSPS.SPSUserVar.enDrvState <= eMS_HOLD)              // im Stop ausgefhrt
      break;
    }

    // Goto-Befehl zu Pos i32_PosMerker[0]
    case 1:
    {
      if (stAppCSPS.SPSCallFunctions.GotoFuncP(stAppCSPS.SPSUserVar.i32_PosVAR_um_01deg[0],eGOTO_um_01deg)==eMS_OK) // Call Goto Function
      {
    	stAppCSPS.SPSUserVar.u8_Dout.ucAllBits |= SET_DO2_PNP_MSK;  // Set Output 2
        stAppCSPS.SPSUserVar.u8_StepChain[0]=0;
    	stAppCSPS.SPSUserVar.u16_Timer5ms[0]=800;                   // 4s -> 5ms x 400
      }
      break;
    }
    // Homing
    case 2:
    {
      if (stAppCSPS.SPSUserVar.enDrvState <= eMS_HOLD)               // warten bis Stop ausgefhrt
      {
    	stAppCSPS.pHomingCntrl->enMode = Homing_DIx_gHi_sm | 3;      // Din3 goes Hi / Single Move
    	stAppCSPS.pHomingCntrl->u16_Timeout_ms = 5000;               // damit wir auch Zeit haben
    	if (stAppCSPS.SPSCallFunctions.HomingPrepFuncP(-50)==eMS_OK) // Call Homing Function
        {
    	  stAppCSPS.SPSUserVar.u8_StepChain[0]++;
        }
      }
      break;
    }
    // Homing-Move end, ..speichern auslsen.. hier ungefragt !!
    case 3:
    {
      if (stAppCSPS.SPSUserVar.enAppState == App_RUN_SPS)           // Homing ist beendet
      {
      	stAppCSPS.pHomingCntrl->enMode = Homing_StoreHome;
      	if (stAppCSPS.SPSCallFunctions.HomingPrepFuncP(0)==eMS_OK) // Call Homing Function
        {
      	  stAppCSPS.SPSUserVar.u8_StepChain[0]=0;
        }
      }
      break;
    }
    // Default Schritt ist quasi Delay Schritt, d.h. nach Ablauf wird Schrittzhler erhht
    default:
    {
      if (stAppCSPS.SPSUserVar.u16_Timer5ms[0]==0)            // wenn Timer bei Null, nchster Schritt
      {
        //stAppCSPS.SPSUserVar.u8_StepChain[0]++;
      }
      break;
    }
  }//..switch end
}

void LOCATEUSER AppCSPS_USER_SEQ_STANDARD_2(void)
{
  mNOP();
}

/************************************************************************************************/
/*!
 *  \brief     Eigentlichen SPS-USER Funktion ( Fehlerablauf ) // Leer Vorlage
 *  \details
 *  \author    Michael Zimmermann
 *  \version   1.0
 *  \date      28.6.2019
 *  \copyright ADLOS
*   \test
*   \param
*   \return
**************************************************************************************************/
void LOCATEUSER AppCSPS_USER_SEQ_ERROR(void)
{
  mNOP();
  mNOP();
  mNOP();
  mNOP();
}





