/*************************************************************************************************/
/*! \file
\ingroup Appl
\brief    Motor-APP / L2 SPS-Integration / USER Seite

- Integration kundenspezifischer CMDs
- bentigt spezielles Linker Skript

CREATED
\date     28.6.2019
\version  1.0
\author   Michael Zimmermann
\remarks  -

<hr>MODIFIED
\date    -
\version -
\author  -
\remarks -

COPYRIGHT  2019, ADLOS AG, FL-9496 Balzers<hr>
**************************************************************************************************/

#include "L0_TA_datatypes_GCC_ARM.h"                                      // uC Include & Std-TRIDONC Type - Definitions

#ifdef xOnlySPSpart
   #include "stm32f0xx.h"
   #include "L0_KannMotDrvTypes.h"
#else
  #include "L0_IncludeFiles.h"                                              // LowLevel Include Dateien
#endif

#include "L2_APPC_SPS_User.h"

// ---- Internal Definitions ----------------------------


// ---- SPS Control-Block ----------------------
volatile const LOCATEUSCNTRL  tAPPCSPS_CNTRLBLK   CNTRLBLKINIT = {
                                                                   .stCntrl.UserStandardFuncPtr_1=AppCSPS_USER_SEQ_STANDARD_1,
                                                                   .stCntrl.UserStandardFuncPtr_2=AppCSPS_USER_SEQ_STANDARD_2,
                                                                   .stCntrl.UserErrorFuncPtr=AppCSPS_USER_SEQ_ERROR,
                                                                   .stCntrl.u32_BlkChkSum=0xFFFFFFFF,
                                                                    #ifdef fFeEXTRAFUNC
                                                                    #ifdef fFeCANRXTX
                                                                   .stCntrl.UserRxEventFuncPtr=AppCSPS_USER_SEQ_CAN_RX_Event,
                                                                    #endif
                                                                   .stCntrl.UserFuncReserved={
                                                                    (tFuncP) 0xFFFFFFFF,
                                                                    (tFuncP) 0xFFFFFFFF,
                                                                    (tFuncP) 0xFFFFFFFF,
                                                                    (tFuncP) 0xFFFFFFFF,
                                                                    (tFuncP) 0xFFFFFFFF,
                                                                    (tFuncP) 0xFFFFFFFF,
                                                                    (tFuncP) 0xFFFFFFFF },
                                                                    #endif
		                                                             .stInfo.su.u16_UserPrgVersion=0x1234,                     // Version
		                                                             .stInfo.su.u16_UserReserved=0xFFFF,                       // Version
		                                                             .stInfo.su.u8_UserPrgTxt={"---- NO ----"}                 // USER Text/ Beschreibung
                                                                 };

// ---- Globale Variabeln lsen -----------------
tAPPCSPS LOCATEVARSPS stAppCSPS;


/************************************************************************************************/
/*!
 *  \brief     SPS-USER Funktion / Demo
 *  \details   This DEMO needs at least Firmware Version V2.5-000 and KannMOTION-Manager 1.7.x.x
 *
 *
 *  USAGE:
 *    stAppCSPS.SPSUserVar.KxxFlexUser.i32_UserVar[0]       : as Helping Position Calculation Value
 *    stAppCSPS.SPSUserVar.UserNonVolatile.i32_UserVar[0]   : Valve 0-Position Offset Absolute in 0.1, NonVolatile
 *
 *  \author    Michael Zimmermann
 *  \version   1.0
 *  \date      6.8.2020
 *  \copyright ADLOS
*   \test
*   \param
*   \return
**************************************************************************************************/
void LOCATEUSER AppCSPS_USER_SEQ_STANDARD_1(void)
{
  // -- First Time, Check NV-Vars if Content is ok, otherwise set to 0
  if (stAppCSPS.SPSUserVar.u8_StepChain[0]==0)
  {
     if ((stAppCSPS.SPSUserVar.UserNonVolatile.i32_UserVar[0] < -7200) || (stAppCSPS.SPSUserVar.UserNonVolatile.i32_UserVar[0] > 7200))
     {
       stAppCSPS.SPSUserVar.UserNonVolatile.i32_UserVar[0] = 0;
     }
     stAppCSPS.SPSUserVar.u8_StepChain[0]++;
  }
}

void LOCATEUSER AppCSPS_USER_SEQ_STANDARD_2(void)
{
}

/************************************************************************************************/
/*!
 *  \brief     Eigentlichen SPS-USER Funktion ( Fehlerablauf ) // Leer Vorlage
 *  \details
 *  \author    Michael Zimmermann
 *  \version   1.0
 *  \date      28.6.2019
 *  \copyright ADLOS
*   \test
*   \param
*   \return
**************************************************************************************************/
void LOCATEUSER AppCSPS_USER_SEQ_ERROR(void)
{
  mNOP();
  mNOP();
  mNOP();
  mNOP();
}

/************************************************************************************************/
/*!
 *  \brief     SPS-USER Function Event is called at CANopen-PDO4 Rx
 *  \details
*   \param    pRxData, Pointer to Data received
*   \param    Datalength, Data count [0..8]
*   \return
**************************************************************************************************/
#ifdef fFeCANRXTX
void LOCATEUSER AppCSPS_USER_SEQ_CAN_RX_Event(tCANDATA* pRxData, UI_8 Datalength)
{
   // .. My Command interpreter
   switch (pRxData->i08_Data[0])
   {
     // CMD=0: Goto-Pos, Target Position is in
     case 00:
     {
        if (Datalength!=8)
        {
           return;
        }
        if (stAppCSPS.SPSCallFunctions.GotoFuncP((SI_32) pRxData->i32_Data[1], eGOTO_um_01deg)==eMS_OK) // Call Goto Function
        {
          stAppCSPS.SPSUserVar.u16_Timer5ms[0]=800;                // 4s -> 5ms x 400
          stAppCSPS.SPSUserVar.u8_StepChain[0]=4;                  // next Step = Delay
        }
       break;
     }
     // CMD=1: Rotate...
     case 01:
     {

       break;
     }
     // CMD=2: Query...
     case 02:
     {
       tCANDATA CanTxData;
       CanTxData.i32_Data[0]=0x01234567;
       CanTxData.i32_Data[1]=0x89ABCDEF;
       if (stAppCSPS.SPSCallFunctions.CANopen_PDO4_Send(&CanTxData,8))
       {
       }
       break;
     }

   }
}
#endif

